import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.net.*;
import java.io.*;
import java.util.*;


public class AiboViewWindowFrame extends JFrame {

    // window components
    private JPanel contentPane;
    private RGBPaintBox paintBoxRGB;
    private BlobPaintBox paintBoxBlob;
    private JButton grab = new JButton("Grab image");
    public JCheckBox segment = new JCheckBox("Segment", false);
    private JButton saveImage = new JButton("Save Image");

    // Construct the frame
    public AiboViewWindowFrame() {
	AiboImage img = null;
	paintBoxRGB = new RGBPaintBox();
	paintBoxBlob = new BlobPaintBox();

	enableEvents(AWTEvent.WINDOW_EVENT_MASK);

	try {
	    init(); // open window
	}
	catch(Exception e) {
	    JOptionPane.showMessageDialog(null, "An error occured and ColorCalibration will now close\nSorry");
	    System.exit(0);
	}


    }

    //Overridden so we can exit when window is closed
    protected void processWindowEvent(WindowEvent e) {
	super.processWindowEvent(e);
	if (e.getID() == WindowEvent.WINDOW_CLOSING) {
	    paintBoxRGB.socket.close();
	    hide();
	}
    }
	
    // Component initialization
    private void init() throws Exception  {
	contentPane = (JPanel) this.getContentPane();
	contentPane.setLayout(null); // absolute positioning
	AiboImage img = null;

	// window size & title
	this.setSize(new Dimension(730, 400));
	this.setTitle("Aibo View");
	paintBoxRGB.setBounds(new Rectangle(5, 2, 355, 288)); // RGB image
	paintBoxBlob.setBounds(new Rectangle(363, 2, 715, 288)); // blob image

	// add the components to the frame
	contentPane.add(paintBoxRGB, null);
	contentPane.add(paintBoxBlob, null);

  	grab.setBounds(new Rectangle(10, 320, 110, 40));
	grab.setToolTipText("Grab image to memory");
  	grab.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    try {
			AiboImage grabbedImage = paintBoxRGB.socket.dataAibo();
			CalibrationCore.aiboImageList.addLast(grabbedImage);
		    }
		    catch(Exception ee){
		    }
		}
	    });
	contentPane.add(grab, null);

	segment.setBounds(new Rectangle(232, 320, 110, 20));
	segment.setToolTipText("Show segmentation");
	contentPane.add(segment, null);

  	saveImage.setBounds(new Rectangle(120, 320, 110, 40));
	saveImage.setToolTipText("Save image to disk");
  	saveImage.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent e) {

		    Calendar time = Calendar.getInstance(); //to get time and date

		    try {
			AiboImage grabbedImage = paintBoxRGB.socket.dataAibo();

			String name = "IMG-" + time.getTime()  + ".kji";
			FileOutputStream out; //declare a file output object
			PrintStream outStream;
			int index = 0;

			byte[] data = new byte[19008];
			
			for(; index < 6336; index++){ // build data
			    data[index] = (new Integer(grabbedImage.yImage[index])).byteValue();
			    data[index + 6336] = (new Integer(grabbedImage.uImage[index])).byteValue();
			    data[index + 12672] = (new Integer(grabbedImage.vImage[index])).byteValue();
			}

			try { // write data to disk
			    out = new FileOutputStream(name);
			    outStream = new PrintStream(out);
			    for(index = 0; index < data.length; index++){
				outStream.write(data[index]);
			    }
			    
			    outStream.close();
			}
			
		
			catch(Exception eeee) {
			    JOptionPane.showMessageDialog(null, "File write error");
			}		    }
		    catch(Exception ee){
		    }
		}
		
	    });
	contentPane.add(saveImage, null);
    }
    
    public class DataAibo {

	private Socket aiboSocket = null;
	private PrintWriter out = null;
	private BufferedReader in = null; 
	private BufferedInputStream YReader = null;
	
	public void dataSocket() throws IOException {
	    
	    // start a socket	    
	    try {
		aiboSocket = new Socket("130.235.21.200", 59180);
		out = new PrintWriter(aiboSocket.getOutputStream(), true);
		YReader = new BufferedInputStream(aiboSocket.getInputStream());
	    } catch (UnknownHostException e) {
		System.err.println("Don't know about host: ");
		System.exit(1);
	    } catch (IOException e) {
		System.err.println("Couldn't get I/O for "
				   + "the connection");
		System.exit(1);
	    }
	}

	public void close() {
	    try {
		aiboSocket.close();
	    }
	    catch(Exception e) {}

	}
	

	public AiboImage dataAibo() throws IOException {
	    
	    try {
		out.flush();
		out.write("X08\0");
		out.flush();

		boolean notDone = true;
		int[] dataY = new int[6336]; // files are 88x72=6336 pixels
		int[] dataU = new int[6336];
		int[] dataV = new int[6336];
		int chY, chU, chV, chC, index;
		
		index = 0;
		
		while (notDone) {  // read all remaining characters in the files
		    
		    if ((index == 6336)) {// end of file
			break;
		    }
		    
		    chY=0;chU=0;chV=0;
		    while (YReader.available() == 0) {}
		    chY = (YReader.read() & 0xff);
		    if (chY > 255)
			chY = 255;
		    
		    while (YReader.available() == 0) {}
		    chU = (YReader.read() & 0xff);
		    if (chU > 255)
			chU = 255;
		    
		    while (YReader.available() == 0) {}
		    chV = (YReader.read() & 0xff);
		    if (chV > 255)
			chV = 255;  		   
		    
		    
		    dataY[index] = chY;
		    dataU[index] = chU;
		    dataV[index] = chV;
		    index++;
		    
		}
		return (new AiboImage(dataY, dataU, dataV, "AiboView")); //done with file
	    }
	    catch(Exception e) {
	    }
	    return null;

	}

 
	
    }

    // inner class to represent the RGB panel
    private class RGBPaintBox extends JPanel {

	private final int X_SIZE = 88;
	private final int Y_SIZE = 72;

	public Color[] colorData;
	public AiboImage image;
	private int pixelSize = 4;

	public DataAibo socket = new DataAibo();

	public RGBPaintBox() {
	    try {
		socket.dataSocket();
	    }
	    catch(Exception e) {
	    }
	}


	// paint rgb image
	public void paint(Graphics gr) {
	    try {
		this.image = socket.dataAibo();
	    }
	    catch(Exception e) {
	    }
	    colorData = image.getRGB();
	    //colorData = image.getBlob();

	    int index = 0;
	    for (int y = 0; y < (Y_SIZE * pixelSize); y=y+pixelSize)
		for (int x = 0; x < (X_SIZE * pixelSize); x=x+pixelSize) {
		    gr.setColor(colorData[index]);
		    gr.fillRect(x, y, pixelSize, pixelSize);
		    index++;
		}

// 	    try {
// 		Thread.sleep(500);
// 	    }
// 	    catch(InterruptedException e) {
// 	    }
	    
	    if (segment.isSelected())
		paintBoxBlob.paint(image);

 	    paintBoxRGB.invalidate();
 	    paintBoxRGB.validate();
 	    paintBoxRGB.repaint();

	}
    }


    // inner class to represent the blob panel
    private class BlobPaintBox extends JPanel {

	private final int X_SIZE = 88;
	private final int Y_SIZE = 72;

	public Color[] colorData;
// 	public AiboImage image;
	private int pixelSize = 4;

	public BlobPaintBox() {
	    //this.image = socket.dataAibo();
	}

	// paint blob image
	public void paint(AiboImage image) {
	    colorData = image.getBlob();

	    Graphics gr = this.getGraphics();

	    //System.out.println("Grapichs: " + gr);

	    int index = 0;
	    for (int y = 0; y < (Y_SIZE * pixelSize); y=y+pixelSize)
		for (int x = 0; x < (X_SIZE * pixelSize); x=x+pixelSize) {
		    gr.setColor(colorData[index]);
		    gr.fillRect(x, y, pixelSize, pixelSize);
		    index++;
		}
	}
    }




}
