import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.filechooser.*;
import java.io.*;


/**
 * Automatic creation of colortabel
 */
public class AutoWindowFrame extends JFrame {
    private JPanel contentPane;
    
    private final int maxPixels = 6336;

    // the histograms
    private int[][] yHistogram = new int[11][maxPixels];
    private int[][] uHistogram = new int[11][maxPixels];
    private int[][] vHistogram = new int[11][maxPixels];
    private int[][] frequency = new int[11][maxPixels];
    private int[][] global = new int[11][maxPixels];
    private int[][] globalFrequency = new int[11][maxPixels];
    private int firstFree = 0;
    private int firstGlobal = 0;
    private boolean useGlobal = true;
    private boolean useLocal = false;

    // Auto options
    private JRadioButton autoOption1 = new JRadioButton("Using current color table"  , true);
    private JRadioButton autoOption2 = new JRadioButton("Using color table on disk", false);
    private ButtonGroup bgroup = new ButtonGroup();
    private JPanel radioPanel = new JPanel();

    //set how sensitive the auto function is with light levels (1 = max, 256 = min)
    private int lightDivider = 8;

    private JButton execute = new JButton("Start");
    private JButton configure = new JButton("Configure");

    private JProgressBar autoProgress = new JProgressBar();

    private int[] imageShare = CalibrationCore.autoPixels; // number of pixels to get from each histogram


    //Construct the frame
    public AutoWindowFrame() {
	enableEvents(AWTEvent.WINDOW_EVENT_MASK);
	try {
	    init();
	}
	catch(Exception e) {
	    JOptionPane.showMessageDialog(null, "An error occured and ColorCalibration will now close\nSorry");
	    System.exit(0);
	}
    }
    
    //Component initialization
    private void init() throws Exception  {
	contentPane = (JPanel) this.getContentPane();
	
	this.setTitle("Automatization");
	this.setSize(290, 200);
	contentPane.setLayout(null);


	bgroup.add(autoOption1);
	bgroup.add(autoOption2);

	radioPanel.setLayout(new GridLayout(3, 1));
	radioPanel.add(autoOption1);
	radioPanel.add(autoOption2);

  	execute.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    if (autoOption1.isSelected())
			autoCurrentTable();
		    else if (autoOption2.isSelected())
			autoOnDiskTable();
		}
	    });

	configure.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    ConfigureAutoWindowFrame frame = new ConfigureAutoWindowFrame();
		    			
		    frame.validate();
		    
		    //Center the window
		    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		    Dimension frameSize = frame.getSize();
		    if (frameSize.height > screenSize.height) {
			frameSize.height = screenSize.height;
		    }
		    if (frameSize.width > screenSize.width) {
			frameSize.width = screenSize.width;
		    }
		    frame.setLocation((screenSize.width - frameSize.width) / 2, (screenSize.height - frameSize.height) / 2);
		    
		    frame.setVisible(true);
		}
	    });

	radioPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Select automatization"));
	
	autoProgress.setDoubleBuffered(true);


	radioPanel.setBounds(new Rectangle(0, 0, 280, 100));
	autoProgress.setBounds(new Rectangle(0, 102, 280, 18));
	execute.setBounds(new Rectangle(170, 122, 110, 40));
	configure.setBounds(new Rectangle(0, 122, 110, 40));

	contentPane.add(radioPanel, null);
	contentPane.add(autoProgress, null);
	contentPane.add(execute, null);
	contentPane.add(configure, null);

    }


    private void autoCurrentTable() {

	int size = CalibrationCore.aiboImageList.size();
	autoProgress.setMaximum(size);

	System.gc();
	
	int[][] pixels = new int[11][maxPixels];

	if (useGlobal)
	    clearGlobal(); // clear out the global histogram

	for (int pic = 0; pic < size; pic++) {

	    //System.out.println("Processing image " + pic);

	    clearHistogram(); // clear local histogram
	    
	    int[] colorIndex = {0,0,0,0,0,0,0,0,0,0,0};
	    AiboImage img = (AiboImage)CalibrationCore.aiboImageList.get(pic); // the image

	    // update progress bar
	    autoProgress.setValue(pic);
	    autoProgress.update(autoProgress.getGraphics());

	    // get the color data
	    int[] imageData = img.getBinBlob();


	    // sort the pixels
	    for (int pos = 0; pos < 6336; pos++) { // each image contains 6336 pixels

		int imagePos = imageData[pos];

		if ((imagePos < 11) && (CalibrationCore.colorsChecked[imagePos])) // if the color is not checked we are not interested...
		    try {
			// save the pixel location int the rigth color array
			pixels[imagePos][colorIndex[imagePos]] = pos;
			
			// build histogram
			buildHistogram((int)img.yImage[pos] / lightDivider, img.uImage[pos], img.vImage[pos], imagePos); // light sensitivity !!!

			colorIndex[imagePos]++;
		    }
		    catch(ArrayIndexOutOfBoundsException e) { // data space wasn't sufficient...
			break;
		    }

	    } // done building histogram

	    
	    // local histogram select
	    for (int col = 0; col < 11; col++) { // for all colors...
		int hist[] = finishHistogram(imageShare[col], col); // need this one for the global as well
		if (useLocal && (CalibrationCore.colorsChecked[col])) { // ...but only if checked!
		    for (int n = 0; n < imageShare[col]; n = n + 3) {
			if (hist[n] == 0)
			    break;
			CalibrationCore.colorTable.addToColorTable(col,(int)hist[n] * lightDivider, hist[n + 1], hist[n + 2]);
		    }
		}		
	    }
	    
	} // done with all the images

	// global histogram select
	if (useGlobal) {
	    for (int col = 0; col < 11; col++) { // for all colors...
		if (CalibrationCore.colorsChecked[col]) { // ...but only if checked!
		    int hist[] = finishGlobal(imageShare[col], col); // get the significant pixels
		    for (int n = 0; n < (imageShare[col] * 3); n = n + 3) {
			if (hist[n] != 0)		    
			    CalibrationCore.colorTable.addToColorTable(col, (int)hist[n] * lightDivider, hist[n + 1], hist[n + 2]);
		    }
		}
	    }
	}

	autoProgress.setValue(size); // ...and we're done.
	System.gc();
    }


    private void autoOnDiskTable() {
	JFileChooser fileChoose = new JFileChooser();
	CCFilter filter = new CCFilter();
	fileChoose.setFileFilter(filter);
	int returnVal = fileChoose.showOpenDialog(null);
	if (returnVal == JFileChooser.APPROVE_OPTION)
	    CalibrationCore.IO.loadColorTable(fileChoose.getSelectedFile());

	autoCurrentTable();
    }

    private class CCFilter extends javax.swing.filechooser.FileFilter {
	public boolean accept(File f) {
	    if (f.isDirectory()) return true;
	    String name = f.getName().toLowerCase();
	    return (name.endsWith("cc"));
	}
	
	public String getDescription() {
	    return "CC files";
	}
    } //end class CCFilter

    // local histogram build
    private void buildHistogram(int y, int u, int v, int colorValue) {
	for (int i = 0; i < maxPixels; i++)
	    if ((yHistogram[colorValue][i] == y) && (uHistogram[colorValue][i] == u) && (vHistogram[colorValue][i] == v)) { // if color already in histogram
		frequency[colorValue][i]++;
		return;
	    }
	if (firstFree < maxPixels) { // if not in histogram, add it at the first free position 
	    yHistogram[colorValue][firstFree] = y;
	    uHistogram[colorValue][firstFree] = u;
	    vHistogram[colorValue][firstFree] = v;
	    firstFree++;
	    frequency[colorValue][firstFree] = 1;
	}
    }
    
    // clears the local histogram
    private void clearHistogram() {

	firstFree = 0;
	
	for (int col = 0; col < 11; col++) { 
	    for (int i = 0; i < maxPixels; i++) {
		yHistogram[col][i] = 0;
		uHistogram[col][i] = 0;
		vHistogram[col][i] = 0;
		frequency[col][i] = 0;
	    }
	}
    }

    // clears the global histogram
    private void clearGlobal() {

	firstGlobal = 0;
	
	for (int col = 0; col < 11; col++) { 
	    for (int i = 0; i < maxPixels; i++) {
		global[col][i] = 0;
		globalFrequency[col][i] = 0;
	    }
	}
    }

    // sort local and build global and return the numberOfValues most significant (local) values
    private int[] finishHistogram(int numberOfValues, int col) {
	numberOfValues = numberOfValues * 3;
	int[] maxFrequency = new int[numberOfValues];

	// build the local histogram
	for (int i = 0; i < (numberOfValues); i = i + 3) {
	    int max = 0;
	    for (int v = 0; v < maxPixels; v++)
		if (frequency[col][v] > max)
		    max = frequency[col][v];
	    //System.out.println("max: " + max);
	    if (max == 0) break; // not in image, do not add
	    for (int v = 0; v < maxPixels; v++)
		if (frequency[col][v] == max) { // any max will do
		    frequency[col][v] = -1;
		    maxFrequency[i] = yHistogram[col][v];
		    maxFrequency[i + 1] = uHistogram[col][v];
		    maxFrequency[i + 2] = vHistogram[col][v];
		    //System.out.println("Local " + CalibrationCore.colorNames[col] + " Y: " + yHistogram[col][v] + " U: " + uHistogram[col][v] + " V: " + vHistogram[col][v] + " max: " + max);
		    break;
		}
	}
        
	// build the global histogram (unsorted)
	//System.out.print("Building global, added ");
	int count = 0;
	for (int i = 0; i < (numberOfValues); i = i + 3) {
	    for (int x = 0; x < maxPixels; x = x + 3)
		if ((maxFrequency[i] == global[col][x]) && (maxFrequency[i+1] == global[col][x+1]) && (maxFrequency[i+2] == global[col][x+2])) {
		    globalFrequency[col][x]++; // pixel found
		    break;
		}
		else if ((firstGlobal < (maxPixels - 3)) && (maxFrequency[i] != 0)) {
		    count++;
		    global[col][firstGlobal] = maxFrequency[i];
		    global[col][firstGlobal+1] = maxFrequency[i+1];
		    global[col][firstGlobal+2] = maxFrequency[i+2];
		    globalFrequency[col][firstGlobal] = 1;
		    
		    firstGlobal = firstGlobal + 3;
		    break;
		}
	}
	//System.out.println(count + " new pixels");

	return maxFrequency;
    }
    
    // sort the global histogram
    private int[] finishGlobal(int numberOfValues, int col) {
	numberOfValues = numberOfValues * 3;
	int[] maxFrequency = new int[numberOfValues];

	// build the histogram
	for (int i = 0; i < (numberOfValues); i = i + 3) {
	    int max = 0;
	    for (int v = 0; v < maxPixels; v = v + 3) // find the most significant pixel
		if (globalFrequency[col][v] > max)
		    max = globalFrequency[col][v];
	    if (max == 0) break; // don't add pixels that don't exist
	    for (int v = 0; v < maxPixels; v = v + 3)
		if (globalFrequency[col][v] == max) { // any max will do
		    //System.out.println("Max: " + max);
		    globalFrequency[col][v] = -1; // delete this max so we won't find it again
		    maxFrequency[i] = global[col][v];
		    maxFrequency[i + 1] = global[col][v+1];
		    maxFrequency[i + 2] = global[col][v+2];
		    //System.out.println("Global " + CalibrationCore.colorNames[col] + " Y: " + global[col][v] + " U: " + global[col][v+1] + " V: " + global[col][v+2] + "  max: " + max);
		    break;
		}
	}
	return maxFrequency;
    }

    // the config window
    private class ConfigureAutoWindowFrame extends JFrame {
	private JPanel contentPane;

	// sliders for all colors
	private JSlider color0 = new JSlider(JSlider.HORIZONTAL, 0, 500, imageShare[0]);
	private JSlider color1 = new JSlider(JSlider.HORIZONTAL, 0, 500, imageShare[1]);
	private JSlider color2 = new JSlider(JSlider.HORIZONTAL, 0, 500, imageShare[2]);
	private JSlider color3 = new JSlider(JSlider.HORIZONTAL, 0, 500, imageShare[3]);
	private JSlider color4 = new JSlider(JSlider.HORIZONTAL, 0, 500, imageShare[4]);
	private JSlider color5 = new JSlider(JSlider.HORIZONTAL, 0, 500, imageShare[5]);
	private JSlider color6 = new JSlider(JSlider.HORIZONTAL, 0, 500, imageShare[6]);
	private JSlider color7 = new JSlider(JSlider.HORIZONTAL, 0, 500, imageShare[7]);
	private JSlider color8 = new JSlider(JSlider.HORIZONTAL, 0, 500, imageShare[8]);
	private JSlider color9 = new JSlider(JSlider.HORIZONTAL, 0, 500, imageShare[9]);
	private JSlider color10 = new JSlider(JSlider.HORIZONTAL, 0, 500, imageShare[10]);

	// labels for all colors
	private JLabel colorButton0 = new JLabel(CalibrationCore.colorNames[0] + " " + imageShare[0]);
	private JLabel colorButton1 = new JLabel(CalibrationCore.colorNames[1] + " " + imageShare[1]);
	private JLabel colorButton2 = new JLabel(CalibrationCore.colorNames[2] + " " + imageShare[2]);
	private JLabel colorButton3 = new JLabel(CalibrationCore.colorNames[3] + " " + imageShare[3]);
	private JLabel colorButton4 = new JLabel(CalibrationCore.colorNames[4] + " " + imageShare[4]);
	private JLabel colorButton5 = new JLabel(CalibrationCore.colorNames[5] + " " + imageShare[5]);
	private JLabel colorButton6 = new JLabel(CalibrationCore.colorNames[6] + " " + imageShare[6]);
	private JLabel colorButton7 = new JLabel(CalibrationCore.colorNames[7] + " " + imageShare[7]);
	private JLabel colorButton8 = new JLabel(CalibrationCore.colorNames[8] + " " + imageShare[8]);
	private JLabel colorButton9 = new JLabel(CalibrationCore.colorNames[9] + " " + imageShare[9]);
	private JLabel colorButton10 = new JLabel(CalibrationCore.colorNames[10] + " " + imageShare[10]);

	// the checkboxes andd a label
	private JCheckBox localHistograms = new JCheckBox("Add colors for each image", useLocal);
	private JCheckBox globalHistograms = new JCheckBox("Add colors for image set", useGlobal);
	private JLabel  checkBoxLabel = new JLabel("How to use histograms");

	public ConfigureAutoWindowFrame() {
	    contentPane = (JPanel)this.getContentPane();
	    JPanel sliderPanel = new JPanel();
	    JPanel checkBoxPanel = new JPanel();
	    
	    this.setTitle("Automatization Config");
	    this.setSize(260, 400);
	    contentPane.setLayout(new BorderLayout());
	    sliderPanel.setLayout(new GridLayout(11,2));
	    checkBoxPanel.setLayout(new GridLayout(3,1));

	    // set bounds for sliders and labels
	    color0.setBounds(new Rectangle(0,0,200,30));
	    color0.setToolTipText("Adjust to select number of pixels of " + CalibrationCore.colorNames[0] + " to add to color table during automatization");
	    color1.setBounds(new Rectangle(0,0,200,30));
	    color1.setToolTipText("Adjust to select number of pixels of " + CalibrationCore.colorNames[1] + " to add to color table during automatization");
	    color2.setBounds(new Rectangle(0,0,200,30));
	    color2.setToolTipText("Adjust to select number of pixels of " + CalibrationCore.colorNames[2] + " to add to color table during automatization");
	    color3.setBounds(new Rectangle(0,0,200,30));
	    color3.setToolTipText("Adjust to select number of pixels of " + CalibrationCore.colorNames[3] + " to add to color table during automatization");
	    color4.setBounds(new Rectangle(0,0,200,30));
	    color4.setToolTipText("Adjust to select number of pixels of " + CalibrationCore.colorNames[4] + " to add to color table during automatization");
	    color5.setBounds(new Rectangle(0,0,200,30));
	    color5.setToolTipText("Adjust to select number of pixels of " + CalibrationCore.colorNames[5] + " to add to color table during automatization");
	    color6.setBounds(new Rectangle(0,0,200,30));
	    color6.setToolTipText("Adjust to select number of pixels of " + CalibrationCore.colorNames[6] + " to add to color table during automatization");
	    color7.setBounds(new Rectangle(0,0,200,30));
	    color7.setToolTipText("Adjust to select number of pixels of " + CalibrationCore.colorNames[7] + " to add to color table during automatization");
	    color8.setBounds(new Rectangle(0,0,200,30));
	    color8.setToolTipText("Adjust to select number of pixels of " + CalibrationCore.colorNames[8] + " to add to color table during automatization");
	    color9.setBounds(new Rectangle(0,0,200,30));
	    color9.setToolTipText("Adjust to select number of pixels of " + CalibrationCore.colorNames[9] + " to add to color table during automatization");
	    color10.setBounds(new Rectangle(0,0,200,30));
	    color10.setToolTipText("Adjust to select number of pixels of " + CalibrationCore.colorNames[10] + " to add to color table during automatization");
	    
	    colorButton0.setBounds(new Rectangle(0,0,50,30));
	    colorButton1.setBounds(new Rectangle(0,0,50,30));
	    colorButton2.setBounds(new Rectangle(0,0,50,30));
	    colorButton3.setBounds(new Rectangle(0,0,50,30));
	    colorButton4.setBounds(new Rectangle(0,0,50,30));
	    colorButton5.setBounds(new Rectangle(0,0,50,30));
	    colorButton6.setBounds(new Rectangle(0,0,50,30));
	    colorButton7.setBounds(new Rectangle(0,0,50,30));
	    colorButton8.setBounds(new Rectangle(0,0,50,30));
	    colorButton9.setBounds(new Rectangle(0,0,50,30));
	    colorButton10.setBounds(new Rectangle(0,0,50,30));

	    sliderPanel.add(color0, null);
	    color0.addChangeListener(new javax.swing.event.ChangeListener() {
		    public void stateChanged(ChangeEvent e) {	
			JSlider source = (JSlider)e.getSource();  // get the slider
			if (!source.getValueIsAdjusting()) {
			    imageShare[0] = source.getValue();  // update
			    colorButton0.setText(CalibrationCore.colorNames[0] + " " + imageShare[0]);
			}
		    }
		});
	    sliderPanel.add(colorButton0, null);

	    sliderPanel.add(color1, null);
	    color1.addChangeListener(new javax.swing.event.ChangeListener() {
		    public void stateChanged(ChangeEvent e) {	
			JSlider source = (JSlider)e.getSource();  // get the slider
			if (!source.getValueIsAdjusting()) {
			    imageShare[1] = source.getValue();  // update
			    colorButton1.setText(CalibrationCore.colorNames[1] + " " + imageShare[1]);
			}
		    }
		});
	    sliderPanel.add(colorButton1, null);

	    sliderPanel.add(color2, null);
	    color2.addChangeListener(new javax.swing.event.ChangeListener() {
		    public void stateChanged(ChangeEvent e) {	
			JSlider source = (JSlider)e.getSource();  // get the slider
			if (!source.getValueIsAdjusting()) {
			    imageShare[2] = source.getValue();  // update
			    colorButton2.setText(CalibrationCore.colorNames[2] + " " + imageShare[2]);
			}
		    }
		});
	    sliderPanel.add(colorButton2, null);

	    sliderPanel.add(color3, null);
	    color3.addChangeListener(new javax.swing.event.ChangeListener() {
		    public void stateChanged(ChangeEvent e) {	
			JSlider source = (JSlider)e.getSource();  // get the slider
			if (!source.getValueIsAdjusting()) {
			    imageShare[3] = source.getValue();  // update
			    colorButton3.setText(CalibrationCore.colorNames[3] + " " + imageShare[3]);
			}
		    }
		});
	    sliderPanel.add(colorButton3, null);

	    sliderPanel.add(color4, null);
	    color4.addChangeListener(new javax.swing.event.ChangeListener() {
		    public void stateChanged(ChangeEvent e) {	
			JSlider source = (JSlider)e.getSource();  // get the slider
			if (!source.getValueIsAdjusting()) {
			    imageShare[4] = source.getValue();  // update
			    colorButton4.setText(CalibrationCore.colorNames[4] + " " + imageShare[4]);
			}
		    }
		});
	    sliderPanel.add(colorButton4, null);

	    sliderPanel.add(color5, null);
	    color5.addChangeListener(new javax.swing.event.ChangeListener() {
		    public void stateChanged(ChangeEvent e) {	
			JSlider source = (JSlider)e.getSource();  // get the slider
			if (!source.getValueIsAdjusting()) {
			    imageShare[5] = source.getValue();  // update
			    colorButton5.setText(CalibrationCore.colorNames[5] + " " + imageShare[5]);
			}
		    }
		});
	    sliderPanel.add(colorButton5, null);

	    sliderPanel.add(color6, null);
	    color6.addChangeListener(new javax.swing.event.ChangeListener() {
		    public void stateChanged(ChangeEvent e) {	
			JSlider source = (JSlider)e.getSource();  // get the slider
			if (!source.getValueIsAdjusting()) {
			    imageShare[6] = source.getValue();  // update
			    colorButton6.setText(CalibrationCore.colorNames[6] + " " + imageShare[6]);
			}
		    }
		});
	    sliderPanel.add(colorButton6, null);

	    sliderPanel.add(color7, null);
	    color7.addChangeListener(new javax.swing.event.ChangeListener() {
		    public void stateChanged(ChangeEvent e) {	
			JSlider source = (JSlider)e.getSource();  // get the slider
			if (!source.getValueIsAdjusting()) {
			    imageShare[7] = source.getValue();  // update
			    colorButton7.setText(CalibrationCore.colorNames[7] + " " + imageShare[7]);
			}
		    }
		});
	    sliderPanel.add(colorButton7, null);

	    sliderPanel.add(color8, null);
	    color8.addChangeListener(new javax.swing.event.ChangeListener() {
		    public void stateChanged(ChangeEvent e) {	
			JSlider source = (JSlider)e.getSource();  // get the slider
			if (!source.getValueIsAdjusting()) {
			    imageShare[8] = source.getValue();  // update
			    colorButton8.setText(CalibrationCore.colorNames[8] + " " + imageShare[8]);
			}
		    }
		});
	    sliderPanel.add(colorButton8, null);

	    sliderPanel.add(color9, null);
	    color9.addChangeListener(new javax.swing.event.ChangeListener() {
		    public void stateChanged(ChangeEvent e) {	
			JSlider source = (JSlider)e.getSource();  // get the slider
			if (!source.getValueIsAdjusting()) {
			    imageShare[9] = source.getValue();  // update
			    colorButton9.setText(CalibrationCore.colorNames[9] + " " + imageShare[9]);
			}
		    }
		});
	    sliderPanel.add(colorButton9, null);

	    sliderPanel.add(color10, null);
	    color10.addChangeListener(new javax.swing.event.ChangeListener() {
		    public void stateChanged(ChangeEvent e) {	
			JSlider source = (JSlider)e.getSource();  // get the slider
			if (!source.getValueIsAdjusting()) {
			    imageShare[10] = source.getValue();  // update
			    colorButton10.setText(CalibrationCore.colorNames[10] + " " + imageShare[10]);
			}
		    }
		});
	    sliderPanel.add(colorButton10, null);

	    checkBoxPanel.add(checkBoxLabel, null);

	    localHistograms.setToolTipText("Check this option to pick the most signficant color values from EACH IMAGE during the automatization process. Usually best results with low values");
	    localHistograms.addChangeListener(new javax.swing.event.ChangeListener() {
		    public void stateChanged(ChangeEvent e) {
			useLocal = localHistograms.isSelected();
		    }
		});

	    globalHistograms.setToolTipText("Check this option to pick the most signficant color values from THE ENTIRE SET of images during the automatization process.");
	    globalHistograms.addChangeListener(new javax.swing.event.ChangeListener() {
		    public void stateChanged(ChangeEvent e) {
			useGlobal = globalHistograms.isSelected();
		    }
		});

	    checkBoxPanel.add(localHistograms, null);
	    checkBoxPanel.add(globalHistograms, null);

	    contentPane.add(sliderPanel, BorderLayout.SOUTH);

	    contentPane.add(checkBoxPanel, BorderLayout.NORTH);

	}


    }
    
}
