
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.filechooser.*;
import java.io.*;
import java.util.LinkedList;

/**
 * The heart of ColorCalibration. Also handles the to window and its menus
 */
public class CalibrationCore extends JFrame {
    private JPanel contentPane;


    // static public variables
    public static ToolboxWindowFrame toolbox;
    public static PictureIndexWindowFrame picIndex;
    public static LinkedList aiboImageList = new LinkedList();
    public static int[] threshold = {8,4,2,3,4,5,6,7,8,0,0};
    public static boolean[] colorsChecked = {true, true, true, true, true, true, true, true, true, false, false};
    public static Color[] colorValue = {new Color(255,105,00), Color.yellow, Color.cyan, Color.pink, Color.green, Color.red, Color.blue, Color.white, new Color(0,160,0), Color.black, Color.lightGray};
    public static String[] colorNames = {"Orange", "Yellow", "SBlue", "Pink", "Green", "Red", "DBlue", "White", "Carpet", "Black", "Silver"};
    public static CalibrationIO IO = new CalibrationIO();
    public static int color = 0;
    public static ColorTable colorTable = new ColorTable();
    public static ColorTableWindowFrame CTWindowFrame = new ColorTableWindowFrame();
    public static boolean useNativeSegmentation = false;
    public static String nativeCode = "";
    public static int[] autoPixels = {20,20,20,20,20,20,20,20,20,0,0};

    private int testindex = 0;

    JFileChooser fileChoose = new JFileChooser();

    // menu items
    private JMenuBar jMenuBar1 = new JMenuBar();
    private JMenu File = new JMenu();
    private JMenuItem Exit = new JMenuItem();
    private JMenuItem Preferences = new JMenuItem();
    private JMenu Images = new JMenu();
    private JMenuItem LoadPic = new JMenuItem();
    private JMenu ColorTable = new JMenu();
    private JMenuItem LoadCT = new JMenuItem();
    private JMenuItem SaveCT = new JMenuItem();
    private JMenu ColorTableMenu = new JMenu();
    private JMenuItem Reset = new JMenuItem();
    private JMenu Help = new JMenu();
    private JMenuItem About = new JMenuItem();
    private JMenuItem HelpText = new JMenuItem();
    private JMenu Option = new JMenu();
    private JMenuItem Automatization = new JMenuItem();
    private JMenu window = new JMenu();
    private JMenuItem showAiboView = new JMenuItem();
    private JMenuItem showTools = new JMenuItem();
    private JMenuItem showCT = new JMenuItem();
    private JMenuItem showIndex = new JMenuItem();

    // Construct the frames
    public CalibrationCore() {
	enableEvents(AWTEvent.WINDOW_EVENT_MASK);
	try {
	    IO.loadPrefs();
	    coreInit();
	    toolboxInit();
	}
	catch(Exception e) {
	    JOptionPane.showMessageDialog(null, "An error occured and ColorCalibration will now close\nSorry");
	    System.exit(0);
	}
    }

    // Start the menu panel
    private void coreInit() throws Exception  {
	contentPane = (JPanel) this.getContentPane();
	contentPane.setLayout(null);
	Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
	Dimension windowSize = this.getSize();
	this.setSize(screenSize.width, 50);
	this.setTitle("Color Calibration");

	// File
	File.setText("File");

	// File - Exit
	Exit.setText("Exit");
	Exit.addActionListener(new java.awt.event.ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		Exit_actionPerformed(e);
	    }
	});

	// File - Preferences
	Preferences.setText("Preferences");
	Preferences.addActionListener(new java.awt.event.ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		PreferencesWindowFrame frame = new PreferencesWindowFrame();
		
		frame.validate();
		
		//place the window to the right
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		Dimension frameSize = toolbox.getSize();
		
		if (frameSize.height > screenSize.height) {
		    frameSize.height = screenSize.height;
		}
		if (frameSize.width > screenSize.width) {
		    frameSize.width = screenSize.width;
		}
		frame.setLocation((screenSize.width - frameSize.width) / 2, (screenSize.height - frameSize.height) / 2);
		
		frame.setVisible(true);	
	    }
	    });


	// File - Images
	Images.setText("Images");

	// File - Images - Load
	LoadPic.setText("Load");
	LoadPic.addActionListener(new java.awt.event.ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		LoadPic_actionPerformed(e);
		picIndexInit();
	    }
	});

	// File - Color Table
	ColorTable.setText("Color Table");

	// File - Color Table - Load
	LoadCT.setText("Load");
	LoadCT.addActionListener(new java.awt.event.ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		loadCT_actionPerformed();
	    }
	});

	// File - Color Table - Save
	SaveCT.setText("Save");
	SaveCT.addActionListener(new java.awt.event.ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		SaveCT_actionPerformed(e);
	    }
	});

	// Color Table
	ColorTableMenu.setText("Color Table");

	// Color Table - Reset
	Reset.setText("Reset");
        Reset.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
		int answer = JOptionPane.showConfirmDialog(null, "Are you sure?");
		if (answer == JOptionPane.YES_OPTION)
		    Reset_actionPerformed(e);
            }
	});

	// Help
	Help.setText("Help");

	// Help - About
	About.setText("About");
        About.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
		String message = "ColorCalibration\nby Carl Axelsson & Jens Torner\n(c) 2003 University of Lund";
		JOptionPane.showMessageDialog(null, message);
            }
	});

	// Help - help
	HelpText.setText("Help");
        HelpText.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
		HelpWindowFrame frame = new HelpWindowFrame();
		
		frame.validate();
		
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		Dimension frameSize = toolbox.getSize();
		
		if (frameSize.height > screenSize.height) {
		    frameSize.height = screenSize.height;
		}
		if (frameSize.width > screenSize.width) {
		    frameSize.width = screenSize.width;
		}
		frame.setLocation((screenSize.width - frameSize.width) / 2, (screenSize.height - frameSize.height) / 2);
		
		frame.setVisible(true);
            }
	});

	// Options
	Option.setText("Options");

	// Options - Automatization
	Automatization.setText("Automatization");
        Automatization.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
		autoInit();
            }
	});


	// Window menu
	window.setText("Window");


	// Window - show toolbox
	showTools.setText("Show Toolbox");
	showTools.addActionListener(new java.awt.event.ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		jMenuItem2_actionPerformed(e);
	    }
	});

	showIndex.setText("Show Image Index");
	showIndex.addActionListener(new java.awt.event.ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		picIndexInit();
	    }
	});	

	showAiboView.setText("Show AiboView");
	showAiboView.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    showAiboView_actionPerformed(e);
		}
	});

	// Color table - Show Color Table
	showCT.setText("Modify");
	showCT.addActionListener(new java.awt.event.ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		jMenuItem3_actionPerformed(e);
	    }
	});



	// Set titles
	jMenuBar1.add(File);
	jMenuBar1.add(ColorTableMenu);
	jMenuBar1.add(Option);
	jMenuBar1.add(window);
	jMenuBar1.add(Help);

	// File menu set-up
	File.add(Images);
	File.add(ColorTable);
	File.addSeparator();
	File.add(Preferences);
	File.add(Exit);
	Images.add(LoadPic);
	ColorTable.add(LoadCT);
	ColorTable.add(SaveCT);

	// Color Table menu set-up
	ColorTableMenu.add(showCT);
	ColorTableMenu.add(Reset);
	//	ColorTableMenu.add(Check);

	// Help menu set-up
	Help.add(About);
	Help.add(HelpText);

	// Options menu set-up
	Option.add(Automatization);

	// Window menu set-up
	window.add(showTools);
	window.add(showIndex);
	window.add(showAiboView); // not implemented yet !!!

	// and add it all to the frame
	this.setJMenuBar(jMenuBar1);
    }

    private void toolboxInit() {

	toolbox = new ToolboxWindowFrame();

	toolbox.validate();

	//place the window to the right
	Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
	Dimension frameSize = toolbox.getSize();

	if (frameSize.height > screenSize.height) {
	    frameSize.height = screenSize.height;
	}
	if (frameSize.width > screenSize.width) {
	    frameSize.width = screenSize.width;
	}
	toolbox.setLocation((screenSize.width - frameSize.width), (screenSize.height - frameSize.height) / 2);

	toolbox.setVisible(true);
    }

    private void picIndexInit() {

	picIndex = new PictureIndexWindowFrame(aiboImageList.size());

	//place the window at the bottom
	Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
	Dimension frameSize = picIndex.getSize();

	if (frameSize.height > screenSize.height) {
	    frameSize.height = screenSize.height;
	}
	if (frameSize.width > screenSize.width) {
	    frameSize.width = screenSize.width;
	}
	picIndex.setLocation(0, screenSize.height - 120 );

	picIndex.validate();

	picIndex.setVisible(true);
    }

    private void autoInit() {

	AutoWindowFrame auto = new AutoWindowFrame();

	// place the window at the bottom
	Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
	Dimension frameSize = auto.getSize();

	if (frameSize.height > screenSize.height) {
	    frameSize.height = screenSize.height;
	}
	if (frameSize.width > screenSize.width) {
	    frameSize.width = screenSize.width;
	}
	auto.setLocation(screenSize.height / 2, screenSize.width / 2 );

	auto.validate();

	auto.setVisible(true);
    }


    //Overridden so we can exit when window is closed
    protected void processWindowEvent(WindowEvent e) {
	super.processWindowEvent(e);
	if (e.getID() == WindowEvent.WINDOW_CLOSING) {
	    IO.savePrefs();
	    System.exit(0);
	}
    }

    private void LoadPic_actionPerformed(ActionEvent e) {
	PGMFilter filter = new PGMFilter(); // get file filter
	KJIFilter filter2 = new KJIFilter();
	fileChoose.setFileFilter(filter2);
	fileChoose.setFileFilter(filter);
	fileChoose.setMultiSelectionEnabled(true);
	fileChoose.showOpenDialog(null);
	File[] files = fileChoose.getSelectedFiles();
	String[] stringFiles = new String[files.length];
	for (int i = 0; i<files.length; i++) {
	    AiboImage img = IO.loadImageYUV(files[i].getAbsolutePath());
	    if (img != null) // in case of a file error, don't add to list
		aiboImageList.addLast(img);
	}
    }

    private void loadCT_actionPerformed() {
	CCFilter filter = new CCFilter();
	fileChoose.setFileFilter(filter);
	int returnVal = fileChoose.showOpenDialog(null);
	if (returnVal == JFileChooser.APPROVE_OPTION)
	    CalibrationCore.IO.loadColorTable(fileChoose.getSelectedFile());
    }


    private void SaveCT_actionPerformed(ActionEvent e) {
	CCFilter filter = new CCFilter();
	fileChoose.setFileFilter(filter);
	int returnVal = fileChoose.showSaveDialog(null);
	String suffix = JOptionPane.showInputDialog(null, "Enter suffix (default _generic)");
	if (suffix != null) {  //if cancel in input dialog
	    if (suffix.equals(""))  //if suffix is change
		suffix = "_generic";	    
	    if (returnVal == JFileChooser.APPROVE_OPTION) 
		CalibrationCore.IO.saveColorTable(fileChoose.getSelectedFile(), suffix);   
	}
    } //end save dialog box

    private void Reset_actionPerformed(ActionEvent e) {
      CalibrationCore.colorTable.resetColorTable(); //reset color table
      for (int i = 0; i<aiboImageList.size(); i++)
	  ((AiboImage)aiboImageList.get(i)).clearBinImage();
    }

    private class PGMFilter extends javax.swing.filechooser.FileFilter {
	public boolean accept(File f) {
	    if (f.isDirectory()) return true;
	    String name = f.getName().toLowerCase();
	    return (name.startsWith("y") && (name.endsWith("pgm")));
	} // end accept

	public String getDescription() {
	    return "Y files";
	}
    }

    private class CCFilter extends javax.swing.filechooser.FileFilter {
	public boolean accept(File f) {
	    if (f.isDirectory()) return true;
	    String name = f.getName().toLowerCase();
	    return (name.endsWith("cc"));
	}
	
	public String getDescription() {
	    return "CC files";
	}
    } //end class CCFilter

    private class KJIFilter extends javax.swing.filechooser.FileFilter {
	public boolean accept(File f) {
	    if (f.isDirectory()) return true;
	    String name = f.getName().toLowerCase();
	    return (name.endsWith("kji"));
	}
	
	public String getDescription() {
	    return "KJI files";
	}
    } //end class KJIFilter

    private void jMenuItem1_actionPerformed(ActionEvent e) {
	boolean packFrame = false;

	AiboImage img = (AiboImage)aiboImageList.get(testindex);
	testindex = testindex++;

	ImageWindowFrame frame = new ImageWindowFrame(img);

	frame.validate();

	//Center the window
	Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
	Dimension frameSize = frame.getSize();
	if (frameSize.height > screenSize.height) {
	    frameSize.height = screenSize.height;
	}
	if (frameSize.width > screenSize.width) {
	    frameSize.width = screenSize.width;
	}
	frame.setLocation((screenSize.width - frameSize.width) / 2, (screenSize.height - frameSize.height) / 2);

	frame.setVisible(true);
    }

    private void jMenuItem2_actionPerformed(ActionEvent e) {
	toolbox.show(); // visa toolbox
    }

    private void jMenuItem3_actionPerformed(ActionEvent e) {
	ColorTableWindowFrame frame = new ColorTableWindowFrame();

	frame.validate();

	//Center the window
	Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
	Dimension frameSize = frame.getSize();
	if (frameSize.height > screenSize.height) {
	    frameSize.height = screenSize.height;
	}
	if (frameSize.width > screenSize.width) {
	    frameSize.width = screenSize.width;
	}
	frame.setLocation((screenSize.width - frameSize.width) / 2, (screenSize.height - frameSize.height) / 2);

	frame.setVisible(true);
    }

    private void showAiboView_actionPerformed(ActionEvent e) {
	
	AiboViewWindowFrame frame = new AiboViewWindowFrame();

	frame.validate();
	
	//Center the window
	Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
	Dimension frameSize = frame.getSize();
	if (frameSize.height > screenSize.height) {
	    frameSize.height = screenSize.height;
	}
	if (frameSize.width > screenSize.width) {
	    frameSize.width = screenSize.width;
	}
	frame.setLocation((screenSize.width - frameSize.width) / 2, (screenSize.height - frameSize.height) / 2);

	frame.setVisible(true);
    }
	
    private void Exit_actionPerformed(ActionEvent e) {
	IO.savePrefs();
	System.exit(0);
    }

 
}



