import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 *  Handles all disk operations
 */
public class CalibrationIO {

    /**
     * Empty constructor
     */
    public CalibrationIO() {
    }

    /**
     * Loads an image from disk and returns the image object
     */
    public AiboImage loadImageYUV(String fileName) {

	File file = new File(fileName);

	String coreName = file.getName();
	coreName = coreName.substring(1, coreName.length());
	String path = file.getAbsolutePath();
	path = path.substring(0, path.length()-coreName.length()-1);
	String extName = coreName.substring(coreName.length()-4, coreName.length());
	extName = extName.toLowerCase();

	String Yname = path + "Y" + coreName;
	String Uname = path + "U" + coreName;
	String Vname = path + "V" + coreName;
	String Cname = path + "I" + coreName;

	FileInputStream YReader = null; // null to avoid compiler warning
	FileInputStream UReader = null;
	FileInputStream VReader = null;
	FileInputStream CReader = null;
	
	// To handle the switch between Window/Unix platforms, ie convert upper/lower case of file names
	if (extName.equals(".pgm")){
	    boolean fileNameOk = false;
	    int fileNamePolicy = 4;
	    while (!fileNameOk) {
		try {
		    YReader = new FileInputStream(new File(Yname));
		    fileNameOk = true;
		}
		catch(Exception e) {
		    if (fileNamePolicy == 4) { // whole name in lower case
			Yname = path + "y" + coreName.toLowerCase();
		    }
		    if (fileNamePolicy == 3) { // whole name in upper case
			Yname = path + "Y" + coreName.toUpperCase();
		    }
		    if (fileNamePolicy == 2) { // extension in upper case
			Yname = path + "Y" + coreName.substring(0, coreName.length()-4) + ".PGM";
		    }
		    if (fileNamePolicy == 1) { // extension in lower case
			Yname = path + "Y" + coreName.substring(0, coreName.length()-4) + ".pgm";
		    }
		    if (fileNamePolicy == 0) { // first letter upper, rest lower
			Yname = path + "Y" + coreName.toLowerCase();
		    }
		    if (fileNamePolicy == -1) {
			JOptionPane.showMessageDialog(null, "Y file error");
			return null;
		    }
		    fileNamePolicy--;
		}
	    }
	    
	    fileNameOk = false;
	    fileNamePolicy = 4;
	    while (!fileNameOk) {
		try {
		    UReader = new FileInputStream(new File(Uname));
		    fileNameOk = true;
		}
		catch(Exception e) {
		    if (fileNamePolicy == 4) { // whole name in lower case
			Uname = path + "u" + coreName.toLowerCase();
		    }
		    if (fileNamePolicy == 3) { // whole name in upper case
			Uname = path + "U" + coreName.toUpperCase();
		    }
		    if (fileNamePolicy == 2) { // extension in upper case
			Uname = path + "U" + coreName.substring(0, coreName.length()-4) + ".PGM";
		    }
		    if (fileNamePolicy == 1) { // extension in lower case
			Uname = path + "U" + coreName.substring(0, coreName.length()-4) + ".pgm";
		    }
		    if (fileNamePolicy == 0) { // first letter upper, rest lower
			Uname = path + "U" + coreName.toLowerCase();
		    }
		    if (fileNamePolicy == -1) {
			JOptionPane.showMessageDialog(null, "U file error");
			return null;
		    }
		    fileNamePolicy--;
		}
	    }
	    
	    fileNameOk = false;
	    fileNamePolicy = 4;
	    while (!fileNameOk) {
		try {
		    VReader = new FileInputStream(new File(Vname));
		    fileNameOk = true;
		}
		catch(Exception e) {
		    if (fileNamePolicy == 4) { // whole name in lower case
			Vname = path + "v" + coreName.toLowerCase();
		    }
		    if (fileNamePolicy == 3) { // whole name in upper case
			Vname = path + "V" + coreName.toUpperCase();
		    }
		    if (fileNamePolicy == 2) { // extension in upper case
			Vname = path + "V" + coreName.substring(0, coreName.length()-4) + ".PGM";
		    }
		    if (fileNamePolicy == 1) { // extension in lower case
			Vname = path + "V" + coreName.substring(0, coreName.length()-4) + ".pgm";
		    }
		    if (fileNamePolicy == 0) { // first letter upper, rest lower
			Vname = path + "V" + coreName.toLowerCase();
		    }
		    if (fileNamePolicy == -1) {
			JOptionPane.showMessageDialog(null, "V file error");
			return null;
		    }
		    fileNamePolicy--;
		}
	    } // end file name upper/lower fix
	}
	else {
	    boolean fileNameOk = false;
	    int fileNamePolicy = 4;
	    while (!fileNameOk) {
		try {
		    CReader = new FileInputStream(new File(Cname));
		    fileNameOk = true;
		}
		catch(Exception e) {
		    if (fileNamePolicy == 4) { // whole name in lower case
			Cname = path + "i" + coreName.toLowerCase();
		    }
		    if (fileNamePolicy == 3) { // whole name in upper case
			Cname = path + "I" + coreName.toUpperCase();
		    }
		    if (fileNamePolicy == 2) { // extension in upper case
			Cname = path + "I" + coreName.substring(0, coreName.length()-4) + ".KJI";
		    }
		    if (fileNamePolicy == 1) { // extension in lower case
			Cname = path + "I" + coreName.substring(0, coreName.length()-4) + ".kji";
		    }
		    if (fileNamePolicy == 0) { // first letter upper, rest lower
			Cname = path + "I" + coreName.toLowerCase();
		    }
		    if (fileNamePolicy == -1) {
			JOptionPane.showMessageDialog(null, "File error");
			return null;
		    }
		    fileNamePolicy--;
		}
	    }   
	}
	boolean notDone = true;
	int[] dataY = new int[6336]; // files are 88x72=6336 pixels
	int[] dataU = new int[6336];
	int[] dataV = new int[6336];
	int chY, chU, chV, chC, index;

	if (extName.equals(".pgm")){
	    try {
		index = 0;
		
		// skip Y header lines, ie read the first four lines
		for(int u = 0; u < 4; u++)
		    while(((int)YReader.read()) != 10);
		
		// skip U header lines
		for(int u = 0; u < 4; u++)
		    while(((int)UReader.read()) != 10);
		
		// skip V header lines
		for(int u = 0; u < 4; u++)
		    while(((int)VReader.read()) != 10);
		
		while (notDone) {  // read all remaining characters in the files
		    chY = YReader.read();

		    chU = UReader.read();
		    chV = VReader.read();
		    if (chY == -1) // end of file
			notDone = false;
		    else { // put read data in array
			dataY[index] = chY;
		    	dataU[index] = chU;
			dataV[index] = chV;
		    	index++;
		    }
		}
		
		// close files
		YReader.close();
		UReader.close();
		VReader.close();
	    }
	    
	    catch(Exception e) {
		JOptionPane.showMessageDialog(null, "File read error");
		return null;
	    }
	    
	    return (new AiboImage(dataY, dataU, dataV, fileName)); // done with file
	}
	else {
	    try {
		index = 0;
		while (notDone) {  // read all remaining characters in the files
		    chC = CReader.read();
		    if (chC == -1) // end of file
			notDone = false;
		    else { // put read data in array
			if ((index > -1) & (index < 6336)){
			    dataY[index] = chC;
			    index++;
			}
			if ((index > 6335) & (index < 12672)){
			    if (index == 6336) // 
				chC = CReader.read(); //take away the added byte from writing all files in one 
			    dataU[index - 6336] = chC;
			    index++;
			}
			if (index > 12671){
			    if (index == 12672) // 
				chC = CReader.read(); //take away the added byte from writing all files in one 
			    dataV[index - 12672] = chC;
			    index++;
			}
		    }
		}
		
		// close files
		CReader.close();
	    }
	    
	    catch(Exception e) {
		JOptionPane.showMessageDialog(null, "File read error");
		return null;
	    }
	    
	    return (new AiboImage(dataY, dataU, dataV, fileName)); //done with file
	}		
    }

    /**
     * Load a colortable
     */
    public void loadColorTable(File fileName) {
	// These are not the user defined color names in order to keep compability with aibo compilations
	String colorNames[] = {"orange", "yellow", "sblue", "pink", "green", "red", "dblue", "white", "carpet", "black", "silver"};
	int NUM_COLORS = 11;
        try {
          FileInputStream fstream = new FileInputStream(fileName); //declare a file inputstream object
          BufferedReader in = new BufferedReader(new InputStreamReader(fstream));

          int symbol = in.read();
          String dataString = null;
          while (symbol != -1) { // while not EOF
	      symbol = in.read();
	      if ((symbol == 47) || (symbol == 35) || (symbol == 13)) {  // if first char equals "/", #, EOL
		  String thres = in.readLine(); //skip to next line
		  if (thres.startsWith("/ Threshold")){ // if threshold is pressent in color table save file
		      String[] UV = {"","","","","","","","","","",""}; //strings containing the threshold values 
		      int UVcount = 0; //to keep track on the threshold value strings 
		      for (int y = 12; y < thres.length(); y++) { // going through the string to find the TH numbers
			  if (!thres.substring(y, y + 1).equals(" "))
			      if (!thres.substring(y, y + 1).equals(","))
				  UV[UVcount] = UV[UVcount] + thres.substring(y, y + 1); //adding number to threshold value  
			      else { //"," marks the end of the threshold value 
				  UVcount++; //next threshold value
			      }
		      }
		      for (int i = 0; i < NUM_COLORS; i++) {
			  int thresValue = Integer.valueOf(UV[i]).intValue(); //making the threshold value strings to integer
			  CalibrationCore.threshold[i] = thresValue; //changing the threshold to the threshold in the load file
		      }
		  }
	      }
	      else {
		  dataString = in.readLine();
		  in.readLine();
		  for (int color = 0; color < NUM_COLORS; color++) {
		      if (isInString(dataString, colorNames[color])) { //checking for what color to use when writing to color table
			  dataString = in.readLine();  //contains min/umax/vmin/vmax values to add to color table
			  for (int light = 0; light < 32; light++) {
			      String[] UV = {"","","",""}; //string containing umin/umax/vmin/vmax
			      int UVcount = 0; 
			      for (int y = 0; y < dataString.length(); y++) { //going through the string to find the umin/umax/vmin/vmax values
				  if (!dataString.substring(y, y + 1).equals(" "))  
				     if (!dataString.substring(y, y + 1).equals(","))
					 UV[UVcount] = UV[UVcount] + dataString.substring(y, y + 1); //adding number to umin or umax or vminor vmax 
				     else { //marks the end of the umin or umax or vmin or vmax value
					 UVcount++; //next value
				     }
			      }
			      int Umi = Integer.valueOf(UV[0]).intValue(); //making the umin value string to integer
			      int Uma = Integer.valueOf(UV[1]).intValue();
			      int Vmi = Integer.valueOf(UV[2]).intValue();
			      int Vma = Integer.valueOf(UV[3]).intValue();
			      CalibrationCore.colorTable.writeColorTable(Umi, Uma, Vmi, Vma, light, color); //chaning the color table to right values
			      dataString = in.readLine(); //read next line
			  }
		      }
		  }
	      }
          }
        }
	
        catch(IOException e)
        {
	    JOptionPane.showMessageDialog(null, "File read error");
	    System.exit(0);

	}

    }

    private boolean isInString(String originalString, String colorString) {

	if (originalString == null) // if EOF
	    return false;
	originalString = originalString.toLowerCase();
	colorString = colorString.toLowerCase();
	int y = 0;
	for (int i = 0; i < originalString.length(); i++) { //going through originalString to find the right color 
          if (y < colorString.length()) {
	      if (originalString.substring(i, i + 1).equals(colorString.substring(y, y + 1))) { //check for match
              y = y + 1;
            }
            else
              y = 0;
          }
        }
        if (y == colorString.length()) //if y match the length of the color then the right color is found
          return true;
        else
          return false;
    }

    /**
     * Save colortable to disk
     */
    public void saveColorTable(File fileName, String suffix) {
	// These are not the user defined color names in order to keep compability with aibo compilations
	String colorNames[] = {"orange", "yellow", "sblue", "pink", "green", "red", "dblue", "white", "carpet", "black", "silver"};
	int NUM_COLORS = 11;
	String colorTableName = fileName.getName();
	String colorTablePath = fileName.getAbsolutePath();

	FileOutputStream out; //declare a file output object
	PrintStream p; //declare a print stream object
	Calendar time = Calendar.getInstance(); //to get time and date

	try {
	    if (colorTableName.endsWith(".cc")) {  //if name ends with .cc
		//create a new file output stream connected to "colorTableName.cc"
		out = new FileOutputStream(colorTablePath);
	    }
	    else
		//create a new file output stream connected to "colorTableName"
		out = new FileOutputStream(colorTablePath + ".cc");

	    //connect print stream to the output stream
	    p = new PrintStream(out);
	    p.println("// Dog color tables"); // writing the string
	    p.println("// Generated by The Colorcalibration Program on " +  time.getTime());
	    p.print("// Threshold: ");
	    for (int i = 0; i < NUM_COLORS; i++)
		p.print (CalibrationCore.threshold[i] + ", "); // writing threshold
	    p.println();
	    p.println();
	    p.println("#define TableSize = 32");
	    p.println();

	    for (int color = 0; color<NUM_COLORS; color++) {
		p.println("const unsigned char " + colorNames[color] + suffix + "[TableSize*4] = \n" + "{");
		for (int light = 0; light<32; light++) {
		    p.println("  " +  CalibrationCore.colorTable.getUmin(light,color) + ", " + CalibrationCore.colorTable.getUmax(light,color) + ", " + CalibrationCore.colorTable.getVmin(light,color) + ", " + CalibrationCore.colorTable.getVmax(light,color) + ","); //writing umin/umax/vmin/vmax for the right color and light
		}
		p.println("};\n");
	    }
	    p.close();
	}
	catch(Exception e){
	    JOptionPane.showMessageDialog(null, "File write error");
	    System.exit(0);

	}
    }

    /**
     * Load preferences file
     */
    public boolean loadPrefs() {
	
	FileInputStream prefsReader;
	
	try {
	    prefsReader = new FileInputStream(new File("prefs"));
	}
	catch (IOException e) {
	    JOptionPane.showMessageDialog(null, "Preferences not found, using defaults");
	    return false;
	}
	
	int i,j;
	
	try {
	    
	    // read color values
	    for (i = 0; i < 11; i++)	
		CalibrationCore.colorValue[i] = new Color(prefsReader.read(), prefsReader.read(), prefsReader.read());
	    
	    // read color names
	    for (i = 0; i < 11; i++) {	
		    StringBuffer name = new StringBuffer("");
		for (j = 0; j < 128; j++) 
		    name.append(new Character((char)prefsReader.read()));

		    String colorName = name.toString();
		    colorName = colorName.trim();
		    CalibrationCore.colorNames[i] = colorName;
	    }
	    
	    // read PAM library
	    StringBuffer name = new StringBuffer("");
	    for (j = 0; j < 128; j++) {
		name.append(new Character((char)prefsReader.read()));
	    }
	    String pamName = ""; 
	    pamName = name.toString();
	    pamName = pamName.trim();
	    CalibrationCore.nativeCode = pamName;

	    // read useNative
	    if (prefsReader.read() == 1)
		CalibrationCore.useNativeSegmentation = true;
	    else
		CalibrationCore.useNativeSegmentation = false;
	}
	catch (IOException e) {
	    JOptionPane.showMessageDialog(null, "Error reading preference file. Using defaults");
	    return false;
	}
	return true;
    }
    
    /**
     * Save the preferences
     */
    public boolean savePrefs() {
	FileOutputStream prefsWriter;
	
	try {
	    prefsWriter = new FileOutputStream(new File("prefs"));
	}
	catch (IOException e) {
	    JOptionPane.showMessageDialog(null, "Error writing preferences");
	    return false;
	}
	
	int i,j;
	
	try {
	    
	    // write color values
	    for (i = 0; i < 11; i++) {	
		prefsWriter.write(CalibrationCore.colorValue[i].getRed()); 
		prefsWriter.write(CalibrationCore.colorValue[i].getGreen()); 
		prefsWriter.write(CalibrationCore.colorValue[i].getBlue()); 
	    }

	    // write color names
	    for (i = 0; i < 11; i++) {	
		StringBuffer name = new StringBuffer(CalibrationCore.colorNames[i]);
		for (j = CalibrationCore.colorNames[i].length(); j < 128; j++)
		    name.append(" "); // fill up with whitespace
		
		for (j = 0; j < 128; j++)
		    prefsWriter.write((int)name.charAt(j));
	    }
	    
	    // write PAM library
	    StringBuffer name = new StringBuffer(CalibrationCore.nativeCode);
	    for (j = CalibrationCore.nativeCode.length(); j < 128; j++)
		name.append(" "); // fill up with whitespace
	    
	    for (j = 0; j < 128; j++)
		prefsWriter.write((int)name.charAt(j));

	    // write useNative
	    if (CalibrationCore.useNativeSegmentation)
		prefsWriter.write(1);
	    else
		prefsWriter.write(0);
	}
	catch (IOException e) {
	    JOptionPane.showMessageDialog(null, "Error writing preference file");
	    return false;
	}
	return true;	
    }
    
}


