/**
 * Handles the color table
 */
public class ColorTable {

    private final int NUM_COLORS = 11; 
    private final int NUM_UNDO = 20;
    private int[][][][] undo = new int[32][NUM_COLORS][20][4]; //undo (light, color,undo,umax/umin/vmax/vmin)
    private int[][] uMin = new int[32][NUM_COLORS]; //contains umin for all light and colors
    private int[][] uMax = new int[32][NUM_COLORS]; //contains umax for all light and colors
    private int[][] vMin = new int[32][NUM_COLORS]; //contains vmin for all light and colors
    private int[][] vMax = new int[32][NUM_COLORS]; //contains vmax for all light and colors
    private int[] index = new int[NUM_COLORS];  //index for undo method

    public ColorTable(){

        for (int i=0; i < NUM_COLORS; i++)
	    index[i] = 1; //resetting the index used in the undo method
        for (int i=0; i<32; i++)
	    for (int j=0; j<NUM_COLORS; j++) {
		this.uMin[i][j] = 255; //resetting all values for umin/umax/vmin/vmax for all colors and light
		this.uMax[i][j] = 0;
		this.vMin[i][j] = 255;
		this.vMax[i][j] = 0;
	    }
	for (int i = 0; i < 32; i++) 
	    for (int j = 0; j < NUM_COLORS; j++) 
		for (int k = 0; k < NUM_UNDO; k++) {
		    undo[i][j][k][0] = 255;  //make backup on umin/umax/vmin/vmax for undo method
		    undo[i][j][k][1] = 0;  
		    undo[i][j][k][2] = 255;  
		    undo[i][j][k][3] = 0; 
		}
    }
    
    /**
     * Returns true if the color specified is in color tabel
     */
    public boolean isInColorTable(int color, int Y, int U, int V) { //checking if pixel is in color table

	int light = Y / 8; // color table is only 32 bit

	if ((U < uMin[light][color]) || (U > uMax[light][color]) || (V < vMin[light][color]) || (V > vMax[light][color])) //checking if pixel is in color table or not
	    return false;
	else
	    return true;
    }

    /**
     * Add a color value to the color table
     */
    public void addToColorTable(int color, int Y, int U, int V) { //adding pixel to color table

	int light = Y / 8; // color table is only 32 bit

	index[color]++;

        if (U < uMin[light][color])  //changing umin/umax/vmin/vmax if necessary
	    uMin[light][color] = U;  //changing umin/umax/vmin/vmax to U/V
	
        if (U > uMax[light][color])
	    uMax[light][color] = U;
	
        if (V < vMin[light][color])
	    vMin[light][color] = V;

        if (V > vMax[light][color])
	    vMax[light][color] = V;
	
	
	if (index[color] == NUM_UNDO) {   // end of undo buffer reached
	    for (int y = 0; y < (NUM_UNDO-2); y++)   // cycle all elements one step back for current color
		for (int i = 0; i < 32; i++) {   // for all light values 
		    undo[i][color][y][0] = undo[i][color][y+1][0]; //one step back for umin/umax/vmin/vmax   
		    undo[i][color][y][1] = undo[i][color][y+1][1];
		    undo[i][color][y][2] = undo[i][color][y+1][2]; 
		    undo[i][color][y][3] = undo[i][color][y+1][3]; 
		}
	    index[color]--;   //adjust undo index
	}
	for (int i = 0; i < 32; i++) {
	    undo[i][color][index[color]][0] = uMin[i][color];  //makeing backup on umin/umax/vmin/vmax for undo method
	    undo[i][color][index[color]][1] = uMax[i][color];  
	    undo[i][color][index[color]][2] = vMin[i][color];  
	    undo[i][color][index[color]][3] = vMax[i][color];  
	}
    }

    /**
     * Reset the color table
     */
    public void resetColorTable() {     //reset all umin/umax/vmin/vmax for all color and all light
      for (int i=0; i<32; i++)
        for (int j=0; j<NUM_COLORS; j++) {
          uMin[i][j] = 255;
          uMax[i][j] = 0;
          vMin[i][j] = 255;
          vMax[i][j] = 0;
	  undo[i][j][index[j]][0] = uMin[i][j];  //makeing backup on umin/umax/vmin/vmax for undo method
	  undo[i][j][index[j]][1] = uMax[i][j];  
	  undo[i][j][index[j]][2] = vMin[i][j];  
	  undo[i][j][index[j]][3] = vMax[i][j];
        }
    }

    /**
     * Undo last change for specified color
     */
    public void undoColorTable(int color) {  //undo pixels for specified color

	if (index[color] != 0)   //trap undo index
	    index[color]--;

	if (index[color] == 0){
	    System.out.println("No more undo!");
	    index[color] = 1;
	}
		
	else
	    for (int i = 0; i < 32; i++) {
		uMin[i][color] = undo[i][color][index[color]][0];  //writing back umin/umax/vmin/vmax
		uMax[i][color] = undo[i][color][index[color]][1];  
		vMin[i][color] = undo[i][color][index[color]][2];  
		vMax[i][color] = undo[i][color][index[color]][3]; 
	    }
    }

    /**
     * Returns the U minimum for specified color and Y (light) value
     */
    public int getUmin(int light,int color) { //geting umin
	return uMin[light][color];
    }
    /**
     * Returns the U maximum for specified color and Y (light) value
     */
    public int getUmax(int light,int color) { //geting umax
	return uMax[light][color];
    }
    /**
     * Returns the V minimum for specified color and Y (light) value
     */
    public int getVmin(int light,int color) { //geting vmin
	return vMin[light][color];
    }
    public int getVmax(int light,int color) { //geting vmax
    /**
     * Returns the V maximum for specified color and Y (light) value
     */
	return vMax[light][color];
    }	 

    /**
     * Set the boundaries for a specified color and light
     */
    public void writeColorTable(int Umin, int Umax, int Vmin, int Vmax, int light, int color) { //writing the right Umin/Umax/Vmin/Vmax values for the loaded color table 
	uMin[light][color] = Umin;
	uMax[light][color] = Umax;
	vMin[light][color] = Vmin;
        vMax[light][color] = Vmax;
    }  
} 


