import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

/**
 * Handles the color tabel window
 */
public class ColorTableWindowFrame extends JFrame {
    public JPanel contentPane;

    // some values
    public int lightValue = 0;
    public int imageWindowLightValue = 0;
    public int U = -1;
    public int V = -1;
    public boolean stopRepainting = false; 
    private int[][][] intensity = new int[255][255][11]; //intensity for a pixel (U, V, color)

    // sliders and labels
    private JSlider vMinSlider = new JSlider(JSlider.VERTICAL, 0, 255, CalibrationCore.colorTable.getVmin(lightValue, CalibrationCore.color));
    private JSlider vMaxSlider = new JSlider(JSlider.VERTICAL, 0, 255, CalibrationCore.colorTable.getVmax(lightValue, CalibrationCore.color));
    private JSlider uMinSlider = new JSlider(JSlider.HORIZONTAL, 0, 255, CalibrationCore.colorTable.getUmin(lightValue, CalibrationCore.color));
    private JSlider uMaxSlider = new JSlider(JSlider.HORIZONTAL, 0, 255, CalibrationCore.colorTable.getUmax(lightValue, CalibrationCore.color));
    public JSlider lightSlider = new JSlider(JSlider.HORIZONTAL, 0, 31, 0);
    private JLabel vMinLabel = new JLabel("vMin");
    private JLabel vMaxLabel = new JLabel("vMax");
    private JLabel uMinLabel = new JLabel("uMin");
    private JLabel uMaxLabel = new JLabel("uMax");
    private JLabel v255Label = new JLabel("v255");
    private JLabel u255Label = new JLabel("u255");
    private JLabel lightLabel = new JLabel("light: 0");

    // Buttons and labels
    public JCheckBox showPixels = new JCheckBox("show pixels", false);
    private JButton updateCT = new JButton("update CT");
    private JButton clearCTLight = new JButton("clear color");
    private JButton close = new JButton ("close");
    private JLabel uMinCoor = new JLabel("umin: ");
    private JLabel uMaxCoor = new JLabel("umax: ");
    private JLabel vMinCoor = new JLabel("vmin: ");
    private JLabel vMaxCoor = new JLabel("vmax: ");
    public JTextField uMinTextField = new JTextField("" + CalibrationCore.colorTable.getUmin(lightValue, CalibrationCore.color)); 
    public JTextField uMaxTextField = new JTextField("" + CalibrationCore.colorTable.getUmax(lightValue, CalibrationCore.color)); 
    public JTextField vMinTextField = new JTextField("" + CalibrationCore.colorTable.getVmin(lightValue, CalibrationCore.color)); 
    public JTextField vMaxTextField = new JTextField("" + CalibrationCore.colorTable.getVmax(lightValue, CalibrationCore.color));
    private JButton currColButt = new JButton("");

    public ColorTablePanel colorTablePanel = new ColorTablePanel();

    //Construct the frame
    public ColorTableWindowFrame() {
	enableEvents(AWTEvent.WINDOW_EVENT_MASK);
	try {
	    init();
	}
	catch(Exception e) {
	    JOptionPane.showMessageDialog(null, "An error occured and ColorCalibration will now close\nSorry");
	    System.exit(0);
	}
    }
    
    
    //Component initialization
    private void init() throws Exception  {
	this.setSize(new Dimension(820, 620));
	this.setTitle("Color Table");

	contentPane = (JPanel) this.getContentPane();
	contentPane.setLayout(null); // absolute positioning
	

	// position the sliders and labels
	vMinSlider.setBounds(new Rectangle(1, 25, 15, 525));
	vMinSlider.setToolTipText("V minimum");
       	vMinSlider.addChangeListener(new javax.swing.event.ChangeListener() {
		public void stateChanged(ChangeEvent e) {

		    int oldVMin = Integer.valueOf(vMinTextField.getText()).intValue();
		    JSlider source = (JSlider)e.getSource();
		    if (!source.getValueIsAdjusting()) {
			int changeVMin = source.getValue();
			if (changeVMin <= vMaxSlider.getValue()) {
			    vMinTextField.setText("" + changeVMin);
			    colorTablePanel.paint(lightValue);
			}
			else
			    vMinSlider.setValue(oldVMin);
		    }
		}
	    });
	vMaxSlider.setBounds(new Rectangle(16, 25, 15, 525));
	vMaxSlider.setToolTipText("V maximum");
       	vMaxSlider.addChangeListener(new javax.swing.event.ChangeListener() {
		public void stateChanged(ChangeEvent e) {

		    int oldVMax = Integer.valueOf(vMaxTextField.getText()).intValue();
		    JSlider source = (JSlider)e.getSource();
		    if (!source.getValueIsAdjusting()) {
			int changeVMax = source.getValue();
			if (changeVMax >= vMinSlider.getValue()) {
			    vMaxTextField.setText("" + changeVMax);
			    colorTablePanel.paint(lightValue);
			}
			else 
			    vMaxSlider.setValue(oldVMax);

		    } 
		}
	    });
	uMinSlider.setBounds(new Rectangle(25, 1, 525, 15));
	uMinSlider.setToolTipText("U minimum");
       	uMinSlider.addChangeListener(new javax.swing.event.ChangeListener() {
		public void stateChanged(ChangeEvent e) {
		   
		    int oldUMin = Integer.valueOf(uMinTextField.getText()).intValue();
		    JSlider source = (JSlider)e.getSource();
		    if (!source.getValueIsAdjusting()) {
			int changeUMin = source.getValue();
			if (changeUMin <= uMaxSlider.getValue()) {
			    uMinTextField.setText("" + changeUMin);
			    colorTablePanel.paint(lightValue);
			}
			else 
			    uMinSlider.setValue(oldUMin);
		    }
		}
	    });	
	uMaxSlider.setBounds(new Rectangle(25, 16, 525, 15));
	uMaxSlider.setToolTipText("U maximum");
       	uMaxSlider.addChangeListener(new javax.swing.event.ChangeListener() {
		public void stateChanged(ChangeEvent e) {

		    int oldUMax = Integer.valueOf(uMaxTextField.getText()).intValue();
		    JSlider source = (JSlider)e.getSource();
		    if (!source.getValueIsAdjusting()) {
			int changeUMax = source.getValue();
			if (changeUMax >= uMinSlider.getValue()) {
			    uMaxTextField.setText("" + changeUMax);
			    colorTablePanel.paint(lightValue);
			} 
			else 	
			    uMaxSlider.setValue(oldUMax);
		    }
		}
	    });
	lightSlider.setBounds(new Rectangle(145,550,250,15));
	lightSlider.setToolTipText("The Y value (light intensity)");
	vMinLabel.setBounds(new Rectangle(2, 552, 45, 15));
	vMaxLabel.setBounds(new Rectangle(2, 567, 45, 15));
	uMinLabel.setBounds(new Rectangle(552, 2, 45, 15));
	uMaxLabel.setBounds(new Rectangle(552, 17, 45 ,15));
	u255Label.setBounds(new Rectangle(521, 541, 55, 15));
	v255Label.setBounds(new Rectangle(548, 32, 55, 15));
	lightLabel.setBounds(new Rectangle(400,550,60,15));
   	lightSlider.addChangeListener(new javax.swing.event.ChangeListener() {
		public void stateChanged(ChangeEvent e) {		    
		    JSlider source = (JSlider)e.getSource();  // get the slider
		    if (!source.getValueIsAdjusting()) {
			int val = source.getValue();
			if (!CalibrationCore.toolbox.displayInCT.isSelected())
			    lightValue = val;
			stopRepainting = true;
			uMinTextField.setText("" + CalibrationCore.colorTable.getUmin(lightValue, CalibrationCore.color));
			uMaxTextField.setText("" + CalibrationCore.colorTable.getUmax(lightValue, CalibrationCore.color));
			vMinTextField.setText("" + CalibrationCore.colorTable.getVmin(lightValue, CalibrationCore.color));
			vMaxTextField.setText("" + CalibrationCore.colorTable.getVmax(lightValue, CalibrationCore.color));
			uMinSlider.setValue(Integer.valueOf(uMinTextField.getText()).intValue());
			uMaxSlider.setValue(Integer.valueOf(uMaxTextField.getText()).intValue());
			vMinSlider.setValue(Integer.valueOf(vMinTextField.getText()).intValue());
			vMaxSlider.setValue(Integer.valueOf(vMaxTextField.getText()).intValue());
			lightLabel.setText("light: " + String.valueOf(lightValue));
			stopRepainting = false;
			colorTablePanel.paint(lightValue);  // update y value
		    }
		}
	    });
	uMinTextField.setBounds(new Rectangle(623,20,35,20));
	uMinTextField.setToolTipText("Umin value for the current color");
	uMinTextField.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    uMinSlider.setValue(Integer.valueOf(uMinTextField.getText()).intValue());
		    uMaxSlider.setValue(Integer.valueOf(uMaxTextField.getText()).intValue());
		    vMinSlider.setValue(Integer.valueOf(vMinTextField.getText()).intValue());
		    vMaxSlider.setValue(Integer.valueOf(vMaxTextField.getText()).intValue());
		    colorTablePanel.paint(lightValue);
		}
	    });
	uMaxTextField.setBounds(new Rectangle(730,20,35,20));
	uMaxTextField.setToolTipText("Umax value for the current color");
	uMaxTextField.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    uMinSlider.setValue(Integer.valueOf(uMinTextField.getText()).intValue());
		    uMaxSlider.setValue(Integer.valueOf(uMaxTextField.getText()).intValue());
		    vMinSlider.setValue(Integer.valueOf(vMinTextField.getText()).intValue());
		    vMaxSlider.setValue(Integer.valueOf(vMaxTextField.getText()).intValue());
		    colorTablePanel.paint(lightValue);
		}
	    });
	vMinTextField.setBounds(new Rectangle(623,55,35,20));
	vMinTextField.setToolTipText("Vmin value for the current color");
	vMinTextField.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    uMinSlider.setValue(Integer.valueOf(uMinTextField.getText()).intValue());
		    uMaxSlider.setValue(Integer.valueOf(uMaxTextField.getText()).intValue());
		    vMinSlider.setValue(Integer.valueOf(vMinTextField.getText()).intValue());
		    vMaxSlider.setValue(Integer.valueOf(vMaxTextField.getText()).intValue());
		    colorTablePanel.paint(lightValue);
		}
	    });
	vMaxTextField.setBounds(new Rectangle(730,55,35,20));
	vMaxTextField.setToolTipText("Vmax value for the current color");
	vMaxTextField.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    uMinSlider.setValue(Integer.valueOf(uMinTextField.getText()).intValue());
		    uMaxSlider.setValue(Integer.valueOf(uMaxTextField.getText()).intValue());
		    vMinSlider.setValue(Integer.valueOf(vMinTextField.getText()).intValue());
		    vMaxSlider.setValue(Integer.valueOf(vMaxTextField.getText()).intValue());
	      	    colorTablePanel.paint(lightValue);
		}
	    });

	// position the buttons
	uMinCoor.setBounds(new Rectangle(623,3,45,15));
	uMaxCoor.setBounds(new Rectangle(730,3,45,15));
	vMinCoor.setBounds(new Rectangle(623,38,45,15));
	vMaxCoor.setBounds(new Rectangle(730,38,45,15));

	showPixels.setBounds(new Rectangle(576,125,110,40));
	showPixels.setToolTipText("Plot pixelvalues from all images");
	showPixels.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent e) {
			colorTablePanel.paint(lightValue);
		    
		}          
	    });
	updateCT.setBounds(new Rectangle(688,80,110,40));
	updateCT.setToolTipText("Update to colortable");
	updateCT.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    CalibrationCore.colorTable.writeColorTable(Integer.valueOf(uMinTextField.getText()).intValue(),Integer.valueOf(uMaxTextField.getText()).intValue(), Integer.valueOf(vMinTextField.getText()).intValue(), Integer.valueOf(vMaxTextField.getText()).intValue(), lightValue, CalibrationCore.color);
		    colorTablePanel.paint(lightValue);
			
		}
	    });

	clearCTLight.setBounds(new Rectangle(576,80,110,40));
	clearCTLight.setToolTipText("Clear U and V values for the current light and color");
	clearCTLight.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    uMinTextField.setText("" + 255);
		    uMaxTextField.setText("" + 0);
		    vMinTextField.setText("" + 255);
		    vMaxTextField.setText("" + 0); 
		    uMinSlider.setValue(Integer.valueOf(uMinTextField.getText()).intValue());
		    uMaxSlider.setValue(Integer.valueOf(uMaxTextField.getText()).intValue());
		    vMinSlider.setValue(Integer.valueOf(vMinTextField.getText()).intValue());
		    vMaxSlider.setValue(Integer.valueOf(vMaxTextField.getText()).intValue());
		}
	    });

	close.setBounds(new Rectangle(688,505,110,40));
  	close.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    hide();
		    CalibrationCore.toolbox.isDisplayInCTVisual = false;
		}
	    });

	currColButt.setBounds(new Rectangle(650,260,80,80));

	// add mouse movment listener
	contentPane.addMouseMotionListener(new java.awt.event.MouseMotionAdapter() {
		public void mouseMoved(MouseEvent e) {
		    int u = ((e.getX() - 32) / 2); //geting right u value
		    int v = ((e.getY() - 32) / 2); //geting right v value

		    // just show colors for the actual CT
		    if (u<0 || u>255 || v<0 || v>255) 
			return;

		    // get color values
		    int r = (int)((lightValue * 8) + 2.028 * (float)(u - 128));
		    int g = (int)((lightValue * 8) + 1.140 * (float)(v - 128));
		    int b = (int)((lightValue * 8) - 0.394 * (float)(u - 128) - 0.581 * (float)(v - 128));

		    // make sure color values are in range 0-255
		    if (r<0) 
			r=0;
		    else if (r>255) 
			r=255;
		    
		    if (g<0) 
			g=0;
		    else if (g>255) 
			g=255;
		    
		    if (b<0) 
			b=0;
		    else if (b>255) 
			b=255;

		    currColButt.setBackground(new Color(r,g,b));
		}
	    });

	
	// test "image"
	colorTablePanel.setBounds(new Rectangle(32, 32, 512, 512));
	
	contentPane.add(vMinSlider, null);
	contentPane.add(vMaxSlider, null);
	contentPane.add(uMinSlider, null);
	contentPane.add(uMaxSlider, null);
	contentPane.add(lightSlider, null);
	contentPane.add(lightLabel, null);
	contentPane.add(uMaxLabel, null);
 	contentPane.add(uMinLabel, null);
	contentPane.add(vMaxLabel, null);
 	contentPane.add(vMinLabel, null);
	contentPane.add(v255Label, null);
	contentPane.add(u255Label, null);
	contentPane.add(colorTablePanel, null);

	contentPane.add(uMinCoor, null);
	contentPane.add(uMinTextField, null);
	contentPane.add(uMaxCoor, null);
	contentPane.add(uMaxTextField, null);
	contentPane.add(vMinCoor, null);
	contentPane.add(vMinTextField, null);
	contentPane.add(vMaxCoor, null);
	contentPane.add(vMaxTextField, null);
	contentPane.add(showPixels, null);
	contentPane.add(updateCT, null);
	contentPane.add(clearCTLight, null);
	contentPane.add(close, null);
	contentPane.add(currColButt, null);
		
	// repaint window information on mouse over
        this.addWindowListener(new java.awt.event.WindowAdapter() {
	    public void windowActivated(WindowEvent e) {
		colorTablePanel.paint(lightValue);  // update y value
	    }
	});
    }
    
    //Overridden so we can exit when window is closed
    protected void processWindowEvent(WindowEvent e) {
	super.processWindowEvent(e);
	if (e.getID() == WindowEvent.WINDOW_CLOSING) {
	    CalibrationCore.toolbox.isDisplayInCTVisual = false;
	    hide();
	}
    }

    
     

    // inner class to draw on the color table panel
    private class ColorTablePanel extends JPanel {
	private final int pixelSize = 2;
	
	public ColorTablePanel() {
	}

        // paint blob image for one specified light level
        public void paint(int yLevel) {
	    
            Graphics gr = this.getGraphics();

            // clear the image
	    gr.setColor(Color.lightGray);
	    gr.fillRect(0, 0, 512, 512);

	    if (showPixels.isSelected()){
	      	if (!stopRepainting)
		    drawPixel(gr);
	    }

	    if (CalibrationCore.toolbox.displayInCT.isSelected()) {
		paintDisplayInCT(gr, CalibrationCore.CTWindowFrame.U, CalibrationCore.CTWindowFrame.V);
		lightSlider.setValue(CalibrationCore.CTWindowFrame.imageWindowLightValue);
		lightValue = CalibrationCore.CTWindowFrame.imageWindowLightValue;
		lightLabel.setText("light: " + String.valueOf(lightValue));
		stopRepainting = true;
		uMinTextField.setText("" + CalibrationCore.colorTable.getUmin(lightValue, CalibrationCore.color));
		uMaxTextField.setText("" + CalibrationCore.colorTable.getUmax(lightValue, CalibrationCore.color));
		vMinTextField.setText("" + CalibrationCore.colorTable.getVmin(lightValue, CalibrationCore.color));
		vMaxTextField.setText("" + CalibrationCore.colorTable.getVmax(lightValue, CalibrationCore.color));
		uMinSlider.setValue(Integer.valueOf(uMinTextField.getText()).intValue());
		uMaxSlider.setValue(Integer.valueOf(uMaxTextField.getText()).intValue());
		vMinSlider.setValue(Integer.valueOf(vMinTextField.getText()).intValue());
		stopRepainting = false;
		vMaxSlider.setValue(Integer.valueOf(vMaxTextField.getText()).intValue());
	    }

	    if (CalibrationCore.toolbox.isColorChanged){
		CalibrationCore.toolbox.isColorChanged = false;
		uMinTextField.setText("" + CalibrationCore.colorTable.getUmin(lightValue, CalibrationCore.color));
		stopRepainting = true; 
		uMaxTextField.setText("" + CalibrationCore.colorTable.getUmax(lightValue, CalibrationCore.color));
		vMinTextField.setText("" + CalibrationCore.colorTable.getVmin(lightValue, CalibrationCore.color));
		vMaxTextField.setText("" + CalibrationCore.colorTable.getVmax(lightValue, CalibrationCore.color));
		uMinSlider.setValue(Integer.valueOf(uMinTextField.getText()).intValue());
		uMaxSlider.setValue(Integer.valueOf(uMaxTextField.getText()).intValue());
		vMinSlider.setValue(Integer.valueOf(vMinTextField.getText()).intValue());
		vMaxSlider.setValue(Integer.valueOf(vMaxTextField.getText()).intValue());
		stopRepainting = false; 
		
	    }
	    		
            for(int col = 10; col > -1; col--) { // to get the color priorities right
		
                // set the color
                gr.setColor(Color.black);
		
		int x,y,he,wi;
		
		if (col == CalibrationCore.color) {
		    // get coords and sizes
		    x = (Integer.valueOf(uMinTextField.getText()).intValue() * pixelSize) - 1;
		    y = (Integer.valueOf(vMaxTextField.getText()).intValue() * pixelSize) - 1; 
		    // max because of orientation
		    wi = ((Integer.valueOf(uMaxTextField.getText()).intValue() - Integer.valueOf(uMinTextField.getText()).intValue()) * pixelSize) + 3;
		    he = ((Integer.valueOf(vMaxTextField.getText()).intValue() - Integer.valueOf(vMinTextField.getText()).intValue()) * pixelSize) + 3;
		}
		else {
		    // get coords and sizes
		    x = (CalibrationCore.colorTable.getUmin(yLevel, col) * pixelSize) - 1;
		    y = (CalibrationCore.colorTable.getVmax(yLevel, col) * pixelSize) - 1;
		    // max because of orientation
		    wi = ((CalibrationCore.colorTable.getUmax(yLevel, col) - CalibrationCore.colorTable.getUmin(yLevel, col)) * pixelSize) + 3;
		    he = ((CalibrationCore.colorTable.getVmax(yLevel, col) - CalibrationCore.colorTable.getVmin(yLevel, col)) * pixelSize) + 3;   
		}

		// draw!
	       	gr.drawRect(x-3, 505-y,wi+6, he+6);
                gr.drawRect(x,508-y,wi,he);
		gr.setColor(CalibrationCore.colorValue[col]);
		gr.drawRect(x-1, 507-y, wi+2, he+2);
	      	gr.drawRect(x-2, 506-y, wi+4, he+4);
            }
        }
	
	// draw all pixels
 	public void drawPixel(Graphics gr) {
	    for (int u = 0; u < 255; u++)
		for (int v = 0; v < 255; v++)
		    for (int c = 0; c < 11; c++)
			intensity[u][v][c] = 0;
	    for (int i = 0; i <  CalibrationCore.aiboImageList.size(); i++) {
		AiboImage img = (AiboImage)CalibrationCore.aiboImageList.get(i);
		for (int pos = 0; pos < 6336; pos++){
		    int Y = img.yImage[pos];
		    int U = img.uImage[pos];
		    int V = img.vImage[pos]; 
		    if (lightValue == (Y/8))
			for (int color = 10; color > -1; color--)
			    if (CalibrationCore.colorTable.isInColorTable(color, Y, U, V)) {			
				intensity[U][V][color]++;
				//	colorTablePanel.drawPixel(U, V, color, intensity[U][V][color]);
				int red = CalibrationCore.colorValue[color].getRed() - (2 * intensity[U][V][color]);
				int green = CalibrationCore.colorValue[color].getGreen() - (2 * intensity[U][V][color]);
				int blue = CalibrationCore.colorValue[color].getBlue() - (2 * intensity[U][V][color]);
				if (red < 0)
				    red = 0;
				if (green < 0)
				    green = 0;
				if (blue < 0)
				    blue = 0;
				Color newColor = new Color(red, green, blue);
				gr.setColor(newColor);
				gr.fillRect((U * pixelSize), 510 - (V * pixelSize),pixelSize,pixelSize);
			    }
		}
	    }
	    
	}

	// just clear the image
        public void paint(Graphics gr) {
	    
	    gr.setColor(Color.lightGray);
	    gr.fillRect(0, 0, 512, 512);
	    for(int col = 10; col > -1; col--) { // to get the color priorities right
                // set the color
                gr.setColor(Color.black);
		int x,y,he,wi;
		if (col == CalibrationCore.color) {
		    // get coords and sizes
		    x = (Integer.valueOf(uMinTextField.getText()).intValue() * pixelSize) - 1;
		    y = (Integer.valueOf(vMaxTextField.getText()).intValue() * pixelSize) - 1; 
		    // max because of orientation
		    wi = ((Integer.valueOf(uMaxTextField.getText()).intValue() - Integer.valueOf(uMinTextField.getText()).intValue()) * pixelSize) + 3;
		    he = ((Integer.valueOf(vMaxTextField.getText()).intValue() - Integer.valueOf(vMinTextField.getText()).intValue()) * pixelSize) + 3;
		}
		else {
		    // get coords and sizes
		    x = (CalibrationCore.colorTable.getUmin(lightValue, col) * pixelSize) - 1;
		    y = (CalibrationCore.colorTable.getVmax(lightValue, col) * pixelSize) - 1;
		    // max because of orientation
		    wi = ((CalibrationCore.colorTable.getUmax(lightValue, col) - CalibrationCore.colorTable.getUmin(lightValue, col)) * pixelSize) + 3;
		    he = ((CalibrationCore.colorTable.getVmax(lightValue, col) - CalibrationCore.colorTable.getVmin(lightValue, col)) * pixelSize) + 3;   
		}
		
		// draw!
		gr.drawRect(x-3, 505-y,wi+6, he+6);
		gr.drawRect(x,508-y,wi,he);
		gr.setColor(CalibrationCore.colorValue[col]);
		gr.drawRect(x-1, 507-y, wi+2, he+2);
		gr.drawRect(x-2, 506-y, wi+4, he+4);
	    }
 
	    if (showPixels.isSelected())
		drawPixel(gr);

	    paintDisplayInCT(gr, CalibrationCore.CTWindowFrame.U, CalibrationCore.CTWindowFrame.V);
	} 
	
	public void paintDisplayInCT(Graphics gr, int u, int v) {

	    gr.setColor(Color.black);
	    gr.fillRect((u * pixelSize), 510 - (v * pixelSize), pixelSize, pixelSize); 
	}
    }
}








    


    
  
    



	      
