import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;


/**
 *
 *  Small little util to convert the tree *.pgm files to one
 *
 *  Usage: FileConverter [files]
 *
 *  where [files] can be any of the Y-, U- or V-files
*/
public class FileConverter {

    private static boolean commandLine = false;
    JFileChooser fileChoose = new JFileChooser();

    public static void main(String args[]) {
	commandLine = true;
	FileConverter fileConverter = new FileConverter(args);
    }


    private class PGMFilter extends javax.swing.filechooser.FileFilter {
	public boolean accept(File f) {
	    if (f.isDirectory()) return true;
	    String name = f.getName().toLowerCase();
	    return (name.startsWith("y") && (name.endsWith("pgm")));
	} // end accept
	
	public String getDescription() {
	    return "Y files";
	}
    }
    

    public FileConverter(String files[]) {
	if (commandLine)
	    System.out.println("FileConverter\n");
	int length = files.length;
	File file = null;
	File[] filefiles = null;
	if (length == 0){
	    PGMFilter filter = new PGMFilter(); // get file filter
	    fileChoose.setFileFilter(filter);
	    fileChoose.setMultiSelectionEnabled(true);
	    fileChoose.showOpenDialog(null);
	    filefiles = fileChoose.getSelectedFiles();  
	    length = filefiles.length;
	}	
	for (int x = 0; x < length; x++) { // for all files
	    if (commandLine){
		if (files.length == 0) {
		    System.out.print("Converting: " + filefiles[x].getName());
		    file = filefiles[x];
		}
		else {
		    System.out.print("Converting: " + files[x]);
		    file = new File(files[x]);
		}
	    }
	    if (convert(file)) { 
		if (commandLine)
		    System.out.println("  done.");
	    }
	}
	System.exit(0);
    }

    /**
     *  Convert files Y[filefilter], U[filename], V[filename] to one file
     *  The file is also compressed using ZLIB compression
     *
     *  Output to disk is compressed bytes[6336*3], first Y, then U and last V
     */
    private boolean convert(File file) {

	
	String coreName = file.getName();
	coreName = coreName.substring(1, coreName.length());
	String path = file.getAbsolutePath();
	path = path.substring(0, path.length()-coreName.length()-1);
	
	String Yname = "Y" + coreName;
	String Uname = "U" + coreName;
	String Vname = "V" + coreName;
	
	
	FileInputStream YReader = null; // null to avoid compiler warning - might not have been initalized
	FileInputStream UReader = null;
	FileInputStream VReader = null;
	
	// To handle the switch between Window/Unix platforms, ie convert upper/lower case of file names
	boolean fileNameOk = false;
	int fileNamePolicy = 4;
	while (!fileNameOk) {
	    try {
		YReader = new FileInputStream(new File(Yname));
		fileNameOk = true;
	    }
	    catch(Exception e) {
		if (fileNamePolicy == 4) { // whole name in lower case
		    Yname = path + "y" + coreName.toLowerCase();
		}
		if (fileNamePolicy == 3) { // whole name in upper case
		    Yname = path + "Y" + coreName.toUpperCase();
		}
		if (fileNamePolicy == 2) { // extension in upper case
		    Yname = path + "Y" + coreName.substring(0, coreName.length()-4) + ".PGM";
		}
		if (fileNamePolicy == 1) { // extension in lower case
		    Yname = path + "Y" + coreName.substring(0, coreName.length()-4) + ".pgm";
		}
		if (fileNamePolicy == 0) { // first letter upper, rest lower
		    Yname = path + "Y" + coreName.toLowerCase();
		}
		if (fileNamePolicy == -1) {
		    JOptionPane.showMessageDialog(null, "V File error");
		    return false;
		}
		fileNamePolicy--;
	    }
	}

	fileNameOk = false;
	fileNamePolicy = 4;
	while (!fileNameOk) {
	    try {
		//System.out.println("Trying "+ Yname);
		UReader = new FileInputStream(new File(Uname));
		fileNameOk = true;
	    }
	    catch(Exception e) {
		if (fileNamePolicy == 4) { // whole name in lower case
		    Uname = path + "u" + coreName.toLowerCase();
		}
		if (fileNamePolicy == 3) { // whole name in upper case
		    Uname = path + "U" + coreName.toUpperCase();
		}
		if (fileNamePolicy == 2) { // extension in upper case
		    Uname = path + "U" + coreName.substring(0, coreName.length()-4) + ".PGM";
		}
		if (fileNamePolicy == 1) { // extension in lower case
		    Uname = path + "U" + coreName.substring(0, coreName.length()-4) + ".pgm";
		}
		if (fileNamePolicy == 0) { // first letter upper, rest lower
		    Uname = path + "U" + coreName.toLowerCase();
		}
		if (fileNamePolicy == -1) {
		    JOptionPane.showMessageDialog(null, "V File error");
		    return false;
		}
		fileNamePolicy--;
	    }
	}

	fileNameOk = false;
	fileNamePolicy = 4;
	while (!fileNameOk) {
	    try {
		//System.out.println("Trying "+ Yname);
		VReader = new FileInputStream(new File(Vname));
		fileNameOk = true;
	    }
	    catch(Exception e) {
		if (fileNamePolicy == 4) { // whole name in lower case
		    Vname = path + "v" + coreName.toLowerCase();
		}
		if (fileNamePolicy == 3) { // whole name in upper case
		    Vname = path + "V" + coreName.toUpperCase();
		}
		if (fileNamePolicy == 2) { // extension in upper case
		    Vname = path + "V" + coreName.substring(0, coreName.length()-4) + ".PGM";
		}
		if (fileNamePolicy == 1) { // extension in lower case
		    Vname = path + "V" + coreName.substring(0, coreName.length()-4) + ".pgm";
		}
		if (fileNamePolicy == 0) { // first letter upper, rest lower
		    Vname = path + "V" + coreName.toLowerCase();
		}
		if (fileNamePolicy == -1) {
		    JOptionPane.showMessageDialog(null, "V File error");
		    return false;
		}
		fileNamePolicy--;
	    }
	} // end file name upper/lower fix
	
	

	boolean notDone = true;
	byte[] data = new byte[6336 * 3]; // files are 88x72 pixels
	int chY, chU, chV, index;

	try {

	    index = 0;

	    // skip Y header lines, ie read the first four lines
	    for(int u = 0; u < 4; u++)
		while(((int)YReader.read()) != 10);

	    // skip U header lines
	    for(int u = 0; u < 4; u++)
		while(((int)UReader.read()) != 10);

	    // skip V header lines
	    for(int u = 0; u < 4; u++)
		while(((int)VReader.read()) != 10);
	    
	    progress();

	    while (notDone) {  // read all remaining characters in the files
		chY = YReader.read();
		chU = UReader.read();
		chV = VReader.read();
		if (chY == -1) // end of file
		    notDone = false;
		else { // put read data in array
		    data[index] = (new Integer(chY)).byteValue();
		    data[index + 6336] = (new Integer(chU)).byteValue();
		    data[index + 12672] = (new Integer(chV)).byteValue();
		    index++;
		}
	    }

	    progress();

	    YReader.close();
	    UReader.close();
	    VReader.close();
	}
	
	catch(Exception e) {
	    JOptionPane.showMessageDialog(null, "File read error");
	    return false;
	}

	progress();



	coreName = path +  coreName.substring(0, coreName.length()-4) + ".kji";
	FileOutputStream out; //declare a file output object
	PrintStream outStream;

	try {
	    out = new FileOutputStream(coreName);
	    outStream = new PrintStream(out);
	    for(index = 0; index < data.length; index++){
		outStream.write(data[index]);
	    }
	    
	    progress();
	  
	    outStream.close();
	}

	catch(Exception e) {
	    JOptionPane.showMessageDialog(null, "File write error");
	}

	return true; // done with file
    }

    // indicate progress to user
    private void progress() {
	if (commandLine)
	    System.out.print(".");
    }

}
 
