import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.math.BigInteger;

/**
 * ImageWindowFrame handles the main image window
 *
 * there can be many instances of ImageWindowFrame open at the same time
 */
public class ImageWindowFrame extends JFrame {

    // window components
    private JPanel contentPane;
    private RGBPaintBox paintBoxRGB;
    private BlobPaintBox paintBoxBlob;
    private JLabel xcoord = new JLabel();
    private JLabel ycoord = new JLabel();
    private JButton forward = new JButton(">>");
    private JButton backward = new JButton("<<");



    //Construct the frame
    public ImageWindowFrame(AiboImage img) {
	paintBoxRGB = new RGBPaintBox(img);
	paintBoxBlob = new BlobPaintBox(img);

	enableEvents(AWTEvent.WINDOW_EVENT_MASK);

	try {
	    init(); // open window
	}
	catch(Exception e) {
	    JOptionPane.showMessageDialog(null, "An error occured and ColorCalibration will now close\nSorry");
	    System.exit(0);
	}
    }

    // Component initialization
    private void init() throws Exception  {
	contentPane = (JPanel) this.getContentPane();
	contentPane.setLayout(null); // absolute positioning

	// window size & title
	this.setSize(new Dimension(642, 402));
	this.setTitle("Image Window");
	
	paintBoxRGB.setBounds(new Rectangle(2, 2, 442, 362)); // RGB image
	paintBoxBlob.setBounds(new Rectangle(452, 2, 628, 146)); // blob image
	
	forward.setBounds(new Rectangle(540, 150, 88, 40)); // forward button
	forward.setToolTipText("Next image");
	backward.setBounds(new Rectangle(452, 150, 88, 40)); // backward button
	backward.setToolTipText("Previous image");
	
	// Coord labels
	xcoord.setText("X-Coord:");
	xcoord.setBounds(new Rectangle(533, 263, 78, 17));
	ycoord.setText("Y-Coord:");
	ycoord.setBounds(new Rectangle(533, 300, 78, 17));

	// repaint window information on mouse over
        this.addWindowListener(new java.awt.event.WindowAdapter() {
		public void windowActivated(WindowEvent e) {
		    paintBoxBlob.image.updateBlobs();
		    paintBoxRGB.paint(paintBoxRGB.getGraphics());
		    paintBoxBlob.paint(paintBoxBlob.getGraphics());
		}
	    });

	// add mouse movment listener
	contentPane.addMouseMotionListener(new java.awt.event.MouseMotionAdapter() {
		public void mouseMoved(MouseEvent e) {
		    contentPane_mouseMoved(e);
		}
	    });

	// add mouse click listener
	contentPane.addMouseListener(new java.awt.event.MouseAdapter() {
		public void mouseClicked(MouseEvent e) {
		    contentPane_mouseClicked(e);
		    if (CalibrationCore.toolbox.displayInCT.isSelected()) {
			int coordx = e.getX();
			int coordy = e.getY();	
			int x = ((coordx - 2) / 5); //geting right x value
			int y = ((coordy - 2) / 5); //geting right y value
			int pos = ((coordy - 2) / 5) * 88 + ((coordx - 2) / 5); //geting possition
			AiboImage img = (AiboImage)CalibrationCore.aiboImageList.get(CalibrationCore.aiboImageList.indexOf(paintBoxRGB.image));
			CalibrationCore.CTWindowFrame.U = img.uImage[pos];
			CalibrationCore.CTWindowFrame.V = img.vImage[pos];
			CalibrationCore.CTWindowFrame.imageWindowLightValue = (img.yImage[pos] / 8); // only uses 32 steps of light

			if (!CalibrationCore.toolbox.isDisplayInCTVisual){
			    ColorTableWindowFrame frame = new ColorTableWindowFrame();
			    
			    frame.validate();
			    
			    //Center the window
			    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
			    Dimension frameSize = frame.getSize();
			    if (frameSize.height > screenSize.height) {
				frameSize.height = screenSize.height;
			    }
			    if (frameSize.width > screenSize.width) {
				frameSize.width = screenSize.width;
			    }
			    frame.setLocation((screenSize.width - frameSize.width) / 2, (screenSize.height - frameSize.height) / 2);
			    
			    frame.setVisible(true);
			    CalibrationCore.toolbox.isDisplayInCTVisual = true;
			}
		    }
		    if (CalibrationCore.toolbox.setBlobView.isSelected()) {
			CalibrationCore.picIndex.scrollPane.invalidate();
			CalibrationCore.picIndex.scrollPane.validate();
			CalibrationCore.picIndex.scrollPane.repaint();
		    }
		}
	    });

	// forward button listener
  	forward.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    // find the next image, if there is one
		    int index = CalibrationCore.aiboImageList.indexOf(paintBoxRGB.image); // find image

		    if (index != -1) {
			index++;
			try {
			    AiboImage i = (AiboImage)CalibrationCore.aiboImageList.get(index);
			    paintBoxRGB.image = i;
			    paintBoxRGB.colorData = i.getRGB(); // since it's only updated once
			    paintBoxBlob.image = i;
			    paintBoxBlob.image.updateBlobs();
			    paintBoxRGB.paint(paintBoxRGB.getGraphics());
			    paintBoxBlob.paint(paintBoxBlob.getGraphics());			
			}
			catch(IndexOutOfBoundsException ee){ // end of list
			}
		    }
		}
	    });

	// backward button listener
  	backward.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    // find the previous image, if there is one
		    int index = CalibrationCore.aiboImageList.indexOf(paintBoxRGB.image);
		    if (index != -1) {
			index--;
			try {
			    AiboImage i = (AiboImage)CalibrationCore.aiboImageList.get(index);
			    paintBoxRGB.image = i;
			    paintBoxRGB.colorData = i.getRGB(); // since it's only updated once
			    paintBoxBlob.image = i;
			    paintBoxBlob.image.updateBlobs();
			    paintBoxRGB.paint(paintBoxRGB.getGraphics());
			    paintBoxBlob.paint(paintBoxBlob.getGraphics());			
			}
			catch(IndexOutOfBoundsException ee){ // start of list
			}
		    }
		}
	    });

	// add the components to the frame
	contentPane.add(paintBoxRGB, null);
	contentPane.add(paintBoxBlob, null);
	contentPane.add(forward, null);
	contentPane.add(backward, null);
	contentPane.add(xcoord, null);
	contentPane.add(ycoord, null);
    }

    // Close window but not app
    protected void processWindowEvent(WindowEvent e) {
	super.processWindowEvent(e);
	if (e.getID() == WindowEvent.WINDOW_CLOSING) {
	    hide();
	}
    }

    // mouse movement, update cursor, and xy-coords if in image area
    private void contentPane_mouseMoved(MouseEvent e) {

	int coordx = e.getX();
	int coordy = e.getY();

	if ((coordx > 2) && (coordx < 442) && (coordy > 2) && (coordy < 362)) { // in RGB area
	    contentPane.setCursor(Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR)); // set cursor to crosshair if in image
	    xcoord.setText("X-Coord:" + (coordx - 2)/5);
	    ycoord.setText("Y-Coord:" + (coordy - 2)/5);
	}
	else if ((coordx > 452) && (coordx < 628) && (coordy > 2) && (coordy < 146)) { // in blob area
	    contentPane.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR)); // set cursor to normal if not in image
	    xcoord.setText("X-Coord:" + (coordx - 452)/2);
	    ycoord.setText("Y-Coord:" + (coordy - 2)/2);
	}
	else
	    contentPane.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR)); // set cursor to normal if not in image
    }

    // mouse click listener
    private void contentPane_mouseClicked(MouseEvent e) {
	int coordx = e.getX();
	int coordy = e.getY();

	int x = ((coordx - 2) / 5); // pixels size in RGB area is 5 
	int y = ((coordy - 2) / 5);

	if ((coordx > 2) && (coordx < 442) && (coordy > 2) && (coordy < 362)) {
	    paintBoxRGB.paintpixel(x, y); // select & repaint
	}

	paintBoxBlob.paint(paintBoxBlob.getGraphics()); // repaint blobs
    }



    // inner class to represent the RGB panel 
    private class RGBPaintBox extends JPanel {
	
	private final int X_SIZE = 88;
	private final int Y_SIZE = 72;

	public Color[] colorData; // public so forward & backward button can acess it
	public AiboImage image; // public for the same reason
	private final int pixelSize = 5; // scale of RGB image

	// constructor
	public RGBPaintBox(AiboImage img) {
	    this.image = img;
	    this.colorData = img.getRGB(); // get the image RGB data - this only need to be done once
	}

	// paint the image from colordata
	public void paint(Graphics gr) {
	    
	    int index = 0;
	    for (int y = 0; y < (Y_SIZE * pixelSize); y=y+pixelSize)
		for (int x = 0; x < (X_SIZE * pixelSize); x=x+pixelSize) {
		    gr.setColor(colorData[index]);
		    gr.fillRect(x, y, pixelSize, pixelSize);
		    index++;
		}
	    
	    boolean[] selectedPixels = image.checkPixelsRGB();
	    gr.setColor(new Color(0,0,0));
	    
	    for (int i = 0; i < 6336; i++)
		if (selectedPixels[i]) {
		    int yy = (int)(i / X_SIZE);
		    int xx = (int)(i - (X_SIZE * yy));
		    gr.fillRect(xx*pixelSize, yy*pixelSize, pixelSize, pixelSize);
		}
	}

	// select & paint
	public void paintpixel(int x, int y) {
	    Graphics gr = this.getGraphics();
	    if (CalibrationCore.toolbox.displayInCT.isSelected()) { // don't draw if CT window display active
		int index = 0;
		for (int yin = 0; yin < (Y_SIZE * pixelSize); yin=yin+pixelSize)
		    for (int xin = 0; xin < (X_SIZE * pixelSize); xin=xin+pixelSize) {
			gr.setColor(colorData[index]);
			gr.fillRect(xin, yin, pixelSize, pixelSize);
			index++;
		    }	
	    }
	    
	    gr.setColor(new Color(0,0,0));
	    int xx = x*pixelSize;
	    int yy = y*pixelSize;
	    gr.fillRect(xx, yy, pixelSize, pixelSize);

	    if (!CalibrationCore.toolbox.displayInCT.isSelected()) { // draw if CT window display not active
	    	boolean[] selectedPixels = image.selectPixel(x, y);
		for (int i = 0; i < 6336; i++)
	    	    if (selectedPixels[i]) {
	    		yy = (int)(i / X_SIZE);
	    		xx = (int)(i - (X_SIZE * yy));
	    		gr.fillRect(xx*pixelSize, yy*pixelSize, pixelSize, pixelSize);
		    }
	    }
	}
    }
    
    // inner class to represent the blob panel
    private class BlobPaintBox extends JPanel {

	private final int X_SIZE = 88;
	private final int Y_SIZE = 72;

	public Color[] colorData;
	public AiboImage image;
	private int pixelSize = 2;

	public BlobPaintBox(AiboImage img) {
	    this.image = img;
	}

	// get and paint blob image
	public void paint(Graphics gr) {
	    colorData = image.getBlob();

	    int index = 0;
	    for (int y = 0; y < (Y_SIZE * pixelSize); y=y+pixelSize)
		for (int x = 0; x < (X_SIZE * pixelSize); x=x+pixelSize) {
		    gr.setColor(colorData[index]);
		    gr.fillRect(x, y, pixelSize, pixelSize);
		    index++;
		}
	}
    }

}
