import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * Handles the image index window
 */
public class PictureIndexWindowFrame extends JFrame {
    public JScrollPane scrollPane;
    
    // Construct the frame
    public PictureIndexWindowFrame(int numberOfPics) {
	
	// window attributes
	setTitle("Image Index");
	setSize((Toolkit.getDefaultToolkit().getScreenSize().width), 120);
	setBackground(Color.gray);

	JPanel topPanel = new JPanel();
 	topPanel.setLayout(new BorderLayout());
	getContentPane().add(topPanel);
	
	// create a scrollable panel to put the images on and make sure it is the right size
	JPanel indexPanel = new JPanel();
	indexPanel.setLayout(new FlowLayout());
	int width = (Toolkit.getDefaultToolkit().getScreenSize().width) - 40;
	int height = (int)((float)(numberOfPics / (width / 80)) * 88) + 50; // just to make sure all images fit
	indexPanel.setPreferredSize(new Dimension(width, height));
	
 	int i = 0;
	
 	for (i = 0; i < numberOfPics; i++) {
	    final AiboImage img = (AiboImage)CalibrationCore.aiboImageList.get(i);
 	    ImageBox ib = new ImageBox(img);
 	    ib.setPreferredSize(new Dimension(88, 72)); // scale 1:1
	    ib.setToolTipText("Single click to display in a new window");
 	    indexPanel.add(ib, null);

	    ib.addMouseListener(new java.awt.event.MouseAdapter() { // display this image in an image window
		    public void mouseClicked(MouseEvent e) {
			ImageWindowFrame frame = new ImageWindowFrame(img);
			
			frame.validate();
			
			//Center the window
			Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
			Dimension frameSize = frame.getSize();
			if (frameSize.height > screenSize.height) {
			    frameSize.height = screenSize.height;
			}
			if (frameSize.width > screenSize.width) {
			    frameSize.width = screenSize.width;
			}
			frame.setLocation((screenSize.width - frameSize.width) / 2, (screenSize.height - frameSize.height) / 2);
			
			frame.setVisible(true);
		    }
		});
 	}	
	
	scrollPane = new JScrollPane();
	scrollPane.getViewport().add(indexPanel);
	topPanel.add(scrollPane, BorderLayout.CENTER);

	this.addWindowListener(new java.awt.event.WindowAdapter() { // repaint the window on focus
		public void windowActivated(WindowEvent e) {
		    scrollPane.invalidate();
		    scrollPane.validate();
		    scrollPane.repaint();
		}
	    });
    }


    // inner class to represent images in the image bar
    private class ImageBox extends JPanel {
    
	AiboImage image;
	
	public ImageBox(AiboImage img) {
	    this.image = img;
	}
		
	public void paint(Graphics gr) {

	    Color[] colorData = null;
	    if (CalibrationCore.toolbox.setBlobView.isSelected()) // draw images as segmented ?
		colorData = image.getBlob(); // ...yes
	    else
		colorData = image.getRGB(); // ...no

	    // draw the image
	    int index = 0;
	    for (int y = 0; y < 72; y++)
		for (int x = 0; x < 88; x++) {
		    gr.setColor(colorData[index]);
		    gr.fillRect(x, y, 1, 1);
		    index++;
		}	
	}
    }
}
