import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.filechooser.*;
import java.io.*;

/**
 * Handles the toolbox window
 */
public class ToolboxWindowFrame extends JFrame {
    
    // these to keep track of color table window
    public boolean isColorChanged = false;
    public boolean isDisplayInCTVisual = false;

    private JPanel contentPane;

    // buttons in the toolbox
    private JButton loadB = new JButton("Load");
    public JCheckBox displayInCT = new JCheckBox("Show In CT", false);
    public JCheckBox setBlobView = new JCheckBox("Show Blobs", false);
    private JButton saveCT = new JButton("SaveCT");
    private JButton undo = new JButton();
    private JLabel thresholdLabel = new JLabel("Threshold");
    private JSlider thresholdSlider = new JSlider(JSlider.HORIZONTAL, 0, 10, 0);
    
    // the colors
    private JButton color0 = new JButton(CalibrationCore.colorNames[0]);
    private JButton color1 = new JButton(CalibrationCore.colorNames[1]);
    private JButton color2 = new JButton(CalibrationCore.colorNames[2]);
    private JButton color3 = new JButton(CalibrationCore.colorNames[3]);
    private JButton color4 = new JButton(CalibrationCore.colorNames[4]);
    private JButton color5 = new JButton(CalibrationCore.colorNames[5]);
    private JButton color6 = new JButton(CalibrationCore.colorNames[6]);
    private JButton color7 = new JButton(CalibrationCore.colorNames[7]);
    private JButton color8 = new JButton(CalibrationCore.colorNames[8]);
    private JButton color9 = new JButton(CalibrationCore.colorNames[9]);
    private JButton color10 = new JButton(CalibrationCore.colorNames[10]);
    


    JFileChooser fileChoose = new JFileChooser();


    //Construct the frame
    public ToolboxWindowFrame() {
	enableEvents(AWTEvent.WINDOW_EVENT_MASK);
	try {
	    init();
	}
	catch(Exception e) {
	    JOptionPane.showMessageDialog(null, "An error occured and ColorCalibration will now close\nSorry");
	    System.exit(0);
	}
    }
    
    //Component initialization
    private void init() throws Exception  {
	contentPane = (JPanel) this.getContentPane();
  	loadB.setBounds(new Rectangle(0, 0, 110, 40));
	loadB.setToolTipText("Load images from disk");
  	loadB.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    loadB_actionPerformed(e);
		}
	    });

  	displayInCT.setBounds(new Rectangle(112, 63, 110, 20));
	displayInCT.setToolTipText("Shows where in color table a pixel value is located");
// 	displayInCT.addActionListener(new java.awt.event.ActionListener() {
// 		public void actionPerformed(ActionEvent e) {
// 		    if (displayInCT.isSelected()) {
// 			modify_actionPerformed(e);
// 		    }
// 		}
// 	    });
        setBlobView.setBounds(new Rectangle(112, 42, 110, 20));
	setBlobView.setToolTipText("Show all images in index window as blobs. Computational intensive!");
	setBlobView.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    CalibrationCore.picIndex.scrollPane.invalidate();
		    CalibrationCore.picIndex.scrollPane.validate();
		    CalibrationCore.picIndex.scrollPane.repaint();
		}
	    });
  	saveCT.setBounds(new Rectangle(0, 42, 110, 40));
	saveCT.setToolTipText("Save color table to disk");
	saveCT.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    SaveCT_actionPerformed(e);
		}
	    });


  	color0.setBounds(new Rectangle(0, 100, 110, 40));
  	color0.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    CalibrationCore.color = 0;
		    setAllButtonsGray();
		    color0.setBackground(CalibrationCore.colorValue[0]);
		    isColorChanged = true;
		}
	    });

  	color1.setBounds(new Rectangle(112, 100, 110, 40));
  	color1.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    CalibrationCore.color = 1;
		    setAllButtonsGray();
		    color1.setBackground(CalibrationCore.colorValue[1]);
		    isColorChanged = true;
		}
	    });

  	color2.setBounds(new Rectangle(0, 142, 110, 40));
  	color2.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    CalibrationCore.color = 2;
		    setAllButtonsGray();
		    color2.setBackground(CalibrationCore.colorValue[2]);
		    isColorChanged = true;
		}
	    });

  	color3.setBounds(new Rectangle(112, 142, 110, 40));
  	color3.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    CalibrationCore.color = 3;
		    setAllButtonsGray();
		    color3.setBackground(CalibrationCore.colorValue[3]);
		    isColorChanged = true;
		}
	    });

  	color4.setBounds(new Rectangle(0, 184, 110, 40));
  	color4.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    CalibrationCore.color = 4;
		    setAllButtonsGray();
		    color4.setBackground(CalibrationCore.colorValue[4]);
		    isColorChanged = true;
		}
	    });

  	color5.setBounds(new Rectangle(112, 184, 110, 40));
  	color5.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    CalibrationCore.color = 5;
		    setAllButtonsGray();
		    color5.setBackground(CalibrationCore.colorValue[5]);
		    isColorChanged = true;
		}
	    });

  	color6.setBounds(new Rectangle(0, 226, 110, 40));
  	color6.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    CalibrationCore.color = 6;
		    setAllButtonsGray();
		    color6.setBackground(CalibrationCore.colorValue[6]);
		    isColorChanged = true;
		}
	    });

  	color7.setBounds(new Rectangle(112, 226, 110, 40));
  	color7.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    CalibrationCore.color = 7;
		    setAllButtonsGray();
		    color7.setBackground(CalibrationCore.colorValue[7]);
		    isColorChanged = true;
		}
	    });

  	color8.setBounds(new Rectangle(0, 268, 110, 40));
  	color8.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    CalibrationCore.color = 8;
		    setAllButtonsGray();
		    color8.setBackground(CalibrationCore.colorValue[8]);
		    isColorChanged = true;
		}
	    });

  	color9.setBounds(new Rectangle(112, 268, 110, 40));
	color9.setForeground(Color.white); // special for black....
  	color9.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    CalibrationCore.color = 9;
		    setAllButtonsGray();
		    color9.setBackground(CalibrationCore.colorValue[9]);
		    isColorChanged = true;
		}
	    });

  	color10.setBounds(new Rectangle(0, 310, 110, 40));
  	color10.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    CalibrationCore.color = 10;
		    setAllButtonsGray();
		    color10.setBackground(CalibrationCore.colorValue[10]);
		    isColorChanged = true;
		}
	    });

	thresholdLabel.setBounds(new Rectangle(137, 310, 85, 15));
				 

	thresholdSlider.setBounds(new Rectangle(112, 325, 110, 40));
	thresholdSlider.setToolTipText("Set threshold for current color");
	thresholdSlider.setMinorTickSpacing(1);
	thresholdSlider.setMajorTickSpacing(5);
	thresholdSlider.setSnapToTicks(true);
	thresholdSlider.setPaintTicks(true);
   	thresholdSlider.addChangeListener(new javax.swing.event.ChangeListener() {
		public void stateChanged(ChangeEvent e) {
		    
		    JSlider source = (JSlider)e.getSource();  // get the slider
		    if (!source.getValueIsAdjusting()) {
			CalibrationCore.threshold[CalibrationCore.color] = source.getValue();  // update threshold			
		    }
		}
	    });

	contentPane.setLayout(null);
	this.setSize(new Dimension(234, 400));
	this.setResizable(false);
	this.setTitle("Tools");



        undo.setBounds(new Rectangle(112, 0, 110, 40));
	undo.setToolTipText("Undo last pixel selection for current color");
	undo.setText("Undo");
	undo.addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    CalibrationCore.colorTable.undoColorTable(CalibrationCore.color);
		    if (setBlobView.isSelected()) {
			CalibrationCore.picIndex.scrollPane.invalidate();
			CalibrationCore.picIndex.scrollPane.validate();
			CalibrationCore.picIndex.scrollPane.repaint();
		    }
		}
	    });

	// start with color 0
	setAllButtonsGray();
	color0.setBackground(CalibrationCore.colorValue[0]);
	CalibrationCore.color = 0;

	// add the buttons
	contentPane.add(loadB, null);
	contentPane.add(undo, null);
	contentPane.add(displayInCT, null);
	contentPane.add(setBlobView, null);
	contentPane.add(saveCT, null);

	// add the colors
	contentPane.add(color0, null);
	contentPane.add(color1, null);
	contentPane.add(color2, null);
	contentPane.add(color3, null);
	contentPane.add(color4, null);
	contentPane.add(color5, null);
	contentPane.add(color6, null);
	contentPane.add(color7, null);
	contentPane.add(color8, null);
	contentPane.add(color9, null);
	contentPane.add(color10, null);

	// add the threshold slider & label
	contentPane.add(thresholdSlider, null);
	contentPane.add(thresholdLabel, null);
    }

    //Overridden so we can exit when window is closed
    protected void processWindowEvent(WindowEvent e) {
	super.processWindowEvent(e);
	if (e.getID() == WindowEvent.WINDOW_CLOSING) {
	    hide();
	}
    }
    
//    void modify_actionPerformed(ActionEvent e) {

//        if (!isDisplayInCTVisual){
// 	   ColorTableWindowFrame frame = new ColorTableWindowFrame();
	   
// 	   frame.validate();
	   
// 	   //Center the window
// 	   Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
// 	   Dimension frameSize = frame.getSize();
// 	   if (frameSize.height > screenSize.height) {
// 	       frameSize.height = screenSize.height;
// 	   }
// 	   if (frameSize.width > screenSize.width) {
// 	       frameSize.width = screenSize.width;
// 	   }
// 	   frame.setLocation((screenSize.width - frameSize.width) / 2, (screenSize.height - frameSize.height) / 2);
	   
// 	   frame.setVisible(true);
// 	   isDisplayInCTVisual = true;
//        }
//    }

    void loadB_actionPerformed(ActionEvent e) {
	PGMFilter filter = new PGMFilter();
	KJIFilter filter2 = new KJIFilter();
	fileChoose.setFileFilter(filter2);
	fileChoose.setFileFilter(filter);
	fileChoose.setMultiSelectionEnabled(true); // do not work on efd!!!
	int returnVal = fileChoose.showOpenDialog(null);
	
	if (returnVal == JFileChooser.APPROVE_OPTION) {
	    File[] files = fileChoose.getSelectedFiles();
	    String[] stringFiles = new String[files.length];
	    for (int i = 0; i<files.length; i++) {
		AiboImage img = CalibrationCore.IO.loadImageYUV(files[i].getAbsolutePath());
		if (img != null)
		    CalibrationCore.aiboImageList.addLast(img);	   
	    }
	}

	
	CalibrationCore.picIndex = new PictureIndexWindowFrame(CalibrationCore.aiboImageList.size());

	//place the window at the bottom
	Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
	Dimension frameSize = CalibrationCore.picIndex.getSize();

	if (frameSize.height > screenSize.height)
	    frameSize.height = screenSize.height;
	
	if (frameSize.width > screenSize.width)
	    frameSize.width = screenSize.width;

	CalibrationCore.picIndex.setLocation(0, screenSize.height - 120 );

	CalibrationCore.picIndex.validate();

	CalibrationCore.picIndex.setVisible(true);

    }

    public void SaveCT_actionPerformed(ActionEvent e) {
	CCFilter filter = new CCFilter();
	fileChoose.setFileFilter(filter);
	int returnVal = fileChoose.showSaveDialog(null);
	String suffix = JOptionPane.showInputDialog(null, "Enter suffix (default _generic)");
	if (suffix != null) {  //if cancel in input dialog
	    if (suffix.equals(""))  //if suffix is change
		suffix = "_generic";	    
	    if (returnVal == JFileChooser.APPROVE_OPTION) 
		CalibrationCore.IO.saveColorTable(fileChoose.getSelectedFile(), suffix);   
	}
    } //end save dialog box

    private class CCFilter extends javax.swing.filechooser.FileFilter {
	public boolean accept(File f) {
	    if (f.isDirectory()) return true;
	    String name = f.getName().toLowerCase();
	    return (name.endsWith("cc"));
	}
	
	public String getDescription() {
	    return "CC files";
	}
    } //end class CCFilter

    private class PGMFilter extends javax.swing.filechooser.FileFilter {
	public boolean accept(File f) {
	    if (f.isDirectory()) return true;
	    String name = f.getName().toLowerCase();
	    return (name.startsWith("y") && name.endsWith("pgm"));
	} // end accept
	
	public String getDescription() {
	    return "Y files";
	}

    } // end class PGMFilter

    private class KJIFilter extends javax.swing.filechooser.FileFilter {
	public boolean accept(File f) {
	    if (f.isDirectory()) return true;
	    String name = f.getName().toLowerCase();
	    return (name.endsWith("kji"));
	}
	
	public String getDescription() {
	    return "KJI files";
	}
    } //end class KJIFilter

    private void setAllButtonsGray() {
	thresholdSlider.setValue(CalibrationCore.threshold[CalibrationCore.color]);

	color0.setBackground(Color.lightGray);
	color1.setBackground(Color.lightGray);
	color2.setBackground(Color.lightGray);
	color3.setBackground(Color.lightGray);
	color4.setBackground(Color.lightGray);
	color5.setBackground(Color.lightGray);
	color6.setBackground(Color.lightGray);
	color7.setBackground(Color.lightGray);
	color8.setBackground(Color.lightGray);
	color9.setBackground(Color.lightGray);
	color10.setBackground(Color.lightGray);
    }


    private void delete_actionPerformed(ActionEvent e) {
	System.exit(0);
    }

    /**
     * Update after preferences
     */
    public void updateColors() {

	// update active button
	if (CalibrationCore.color == 0)
	    color0.setBackground(CalibrationCore.colorValue[0]);
	if (CalibrationCore.color == 1)
	    color1.setBackground(CalibrationCore.colorValue[1]);
	if (CalibrationCore.color == 2)
	    color2.setBackground(CalibrationCore.colorValue[2]);
	if (CalibrationCore.color == 3)
	    color3.setBackground(CalibrationCore.colorValue[3]);
	if (CalibrationCore.color == 4)
	    color4.setBackground(CalibrationCore.colorValue[4]);
	if (CalibrationCore.color == 5)
	    color5.setBackground(CalibrationCore.colorValue[5]);
	if (CalibrationCore.color == 6)
	    color6.setBackground(CalibrationCore.colorValue[6]);
	if (CalibrationCore.color == 7)
	    color7.setBackground(CalibrationCore.colorValue[7]);
	if (CalibrationCore.color == 8)
	    color8.setBackground(CalibrationCore.colorValue[8]);
	if (CalibrationCore.color == 9)
	    color9.setBackground(CalibrationCore.colorValue[9]);
	if (CalibrationCore.color == 10)
	    color10.setBackground(CalibrationCore.colorValue[10]);

	// update names
	color0.setText(CalibrationCore.colorNames[0]);
	color1.setText(CalibrationCore.colorNames[1]);
	color2.setText(CalibrationCore.colorNames[2]);
	color3.setText(CalibrationCore.colorNames[3]);
	color4.setText(CalibrationCore.colorNames[4]);
	color5.setText(CalibrationCore.colorNames[5]);
	color6.setText(CalibrationCore.colorNames[6]);
	color7.setText(CalibrationCore.colorNames[7]);
	color8.setText(CalibrationCore.colorNames[8]);
	color9.setText(CalibrationCore.colorNames[9]);
	color10.setText(CalibrationCore.colorNames[10]);

    }
}
