
/* Daniel Karlsson, sep 2003             */
/* Keyboard and Joystick control of AIBO */
 
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include "misc.h"
#include "joystick.h"
#include "aibo.h"

#define BUFSIZE  512

struct sockaddr_in  addr;
struct hostent      *hp;
int    fd;

void send_command(char *command) {
  int    len;
  char   buf[20000];
  int    ret;
  
  sprintf(buf,"%s\n",command);
  len = strlen(buf) + 1;
  
  ret = write(fd, buf, len);
  if (ret != len) {
    perror("write");
    close(fd);
    exit(1);
  }
  // Clarification that everything went ok
  ret = read(fd, buf, len);
  if (ret != len) {
    perror("read");
    close(fd);
    exit(1);
  }
  
  //buf[ret] = '\0';
  //printf("From AIBO: %s\n",buf);
  //delay(1);
}

void joystick(void) {
  printf("Start redaing from joystick\n");
  read_joystick();
  printf("\n\nQuiting joystick mode...\n");
  delay(1);
}

void keyboard(void) {
  bool quit = FALSE;
  char command;
  while (!quit) {
    clrscr();
    printf("Use numpad to control\n)");
    printf("Type 'q' to quit\n\n");
    command = getKey();
    switch(command) {
    case '0'       : send_command("K00"); break;
    case '1'       : send_command("K01"); break;
    case '2'       : send_command("K02"); break;
    case '3'       : send_command("K03"); break;
    case '4'       : send_command("K04"); break;
    case '5'       : send_command("K05"); break;
    case '6'       : send_command("K06"); break;
    case '7'       : send_command("K07"); break;
    case '8'       : send_command("K08"); break;
    case '9'       : send_command("K09"); break;
    case 'z'       : send_command("K10"); break;
    case 'x'       : send_command("K11"); break;
    case 'c'       : send_command("K12"); break;
    case 'v'       : send_command("K13"); break;
    case 'b'       : send_command("K14"); break;
    case 'n'       : send_command("K15"); break;
    case 'm'       : send_command("K16"); break;
    case 'p'       :  send_command("X") ; break;
    case 'q'       : send_command("K05"); quit = TRUE; break;
    default        : ;
    }
  }
}

main(int argc, char **argv) {
  
  open_joystick();
  
  // Make connection to AIBO
  if ((fd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
    perror("socket");
    exit(1);
  }
  if ((hp = gethostbyname(HOSTNAME)) == NULL) {
    perror("gethostbyname");
    exit(1);
  }
  memset((void*)&addr, 0, sizeof(addr));
  memcpy(&addr.sin_addr, hp->h_addr, hp->h_length);
  addr.sin_family = AF_INET;
  addr.sin_port = htons(PORT);
  printf("Host: %s\nPort: %d\n",HOSTNAME,PORT);
  while (1) {
    if (connect(fd, (struct sockaddr *)&addr, sizeof(addr)) < 0)
      delay(1);
    else
      break;
  }

  bool quit = FALSE;
  char choise[50];
  
  while(!quit) {
    clrscr();
    printf("Welcome!\n");
    printf("Please choose action:\n");
    printf("1.\tKeyboard control\n");
    printf("2.\tJoystick control\n");
    printf("3.\tQuit\n\n\\> ");
    fgets(choise,sizeof(choise),stdin);
    switch(choise[0]) {
    case '1' : keyboard(); break;
    case '2' : joystick(); break;
    case '3' : quit = TRUE; break;
    default  : printf("Please choose one of the above..."); delay(1);
    }
  }
  close(fd);
  exit(0);
}
