
/* Daniel Karlsson, sep 2003             */
/* Keyboard and Joystick control of AIBO */

#include <sys/ioctl.h>
#include <sys/time.h>
#include <sys/types.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>

#include <linux/joystick.h>
#include "joystick.h"
#include "misc.h"

int fd2;
unsigned char axes = 2;
unsigned char buttons = 2;
int *axis;
int *button;

void open_joystick() {
  int version = 0x000800;
  char name[128] = "Unknown";

  if ((fd2 = open(JOYSTICKPATH, O_RDONLY)) < 0) {
    perror("Error opening joystick");
    exit(1);
  }
  
  ioctl(fd2, JSIOCGVERSION, &version);
  ioctl(fd2, JSIOCGAXES, &axes);
  ioctl(fd2, JSIOCGBUTTONS, &buttons);
  ioctl(fd2, JSIOCGNAME(128), name);
  
  printf("Joystick (%s) has %d axes and %d buttons. Driver version is %d.%d.%d.\n",
	 name, axes, buttons, version >> 16, (version >> 8) & 0xff, version & 0xff);
 
}

void read_joystick() {
  char temp_send_data[256];
  sprintf(temp_send_data,"J0#0#0#0#0#0#0#0#0#0#0#0#0#0");
  int i;
  int init_counter = 0;
  struct js_event js;
  axis = calloc(axes, sizeof(int));
  button = calloc(buttons, sizeof(char));
  while (1) {
    if (read(fd2, &js, sizeof(struct js_event)) != sizeof(struct js_event)) {
      perror("\nError reading joystick");
      exit (1);
    }
    switch(js.type & ~JS_EVENT_INIT) {
    case JS_EVENT_BUTTON:
      button[js.number] = js.value;
      break;
    case JS_EVENT_AXIS:
      axis[js.number] = js.value;
      break;
    }
    if (init_counter < 26) // Remove the number of initiateing events
      init_counter++;
    else {
      int i = 0;
      char tmp[6];
      int t[axes];
      
      // Make +- 32000 to +-3
      for (i = 0; i < axes; i++) {
	sprintf(tmp,"%6d",axis[i]);
	sscanf(tmp,"%d",&t[i]);
	t[i] /= 10000;
      }
      char send_data[256];
      sprintf(send_data,"J%d#%d#%d#%d#%d#%d#%d#%d#%d#%d#%d#%d#%d#%d",
		   t[0],t[1],t[4],t[5],t[6],t[7],
		   button[0],button[1],button[2],button[3],button[4],button[5],button[6],button[7]);

      if (strcmp(temp_send_data,send_data)) {
	//printf("Sending: %s\n",send_data);
	strcpy(temp_send_data,send_data);
	send_command(send_data);
      }

      if (button[7]) // Quit button (button # 8)
	break;
    }
  }
}
