//
// Copyright 2002 Sony Corporation 
//
// Permission to use, copy, modify, and redistribute this software for
// non-commercial use is hereby granted.
//
// This software is provided "as is" without warranty of any kind,
// either expressed or implied, including but not limited to the
// implied warranties of fitness for a particular purpose.
//

#ifndef Server_h_DEFINED
#define Server_h_DEFINED

#include <OPENR/OObject.h>
#include <OPENR/OSubject.h>
#include <OPENR/OObserver.h>
#include "TCPConnection.h"
#include "ServerConfig.h"
#include "def.h"

#define SIZE_OF_PACKAGE 528

static const char* const FBK_LOCATOR = "PRM:/r1/c1/c2/c3/i1-FbkImageSensor:F1";

class Server : public OObject {
public:

    Server();
    virtual ~Server() {}

    OSubject*   subject[numOfSubject];
    OObserver*  observer[numOfObserver];     

    virtual OStatus DoInit    (const OSystemEvent& event);
    virtual OStatus DoStart   (const OSystemEvent& event);
    virtual OStatus DoStop    (const OSystemEvent& event);
    virtual OStatus DoDestroy (const OSystemEvent& event);

    void Ready (const OReadyEvent& event);
    void PictureNotify(const ONotifyEvent& event);

    void ListenCont  (ANTENVMSG msg);
    void SendCont    (ANTENVMSG msg);
    void ReceiveCont (ANTENVMSG msg);
    void CloseCont   (ANTENVMSG msg);

private:
    OStatus Listen  (int index);
    OStatus Send    (int index);
    OStatus Receive (int index);
    OStatus Close   (int index);
    OStatus InitTCPConnection (int index);

    void OpenImagePrimitive();
    void SetSendData    (int index);
    void SetReceiveData (int index);

    int pictureSend;
    int total_sent;
    
    //byte mybuffer[20000];
    OPrimitiveID fbkID;

    antStackRef    ipstackRef;
    TCPConnection  connection[SERVER_CONNECTION_MAX];

    bool readySend;

};

#endif // Server_h_DEFINED
