//=======================================================================================
//
// AiboImage.c
//
// Routines for comunication between Segment and ColorCalibration
//
// Author: Jens Trner, dat00jet@ludat.lth.se
// Copyright (C) 2003 Lund University and Team Sweden, Sweden
// All Rights Reserved
//=======================================================================================

#include <jni.h>
#include "AiboImage.h"

/* 
kompileringsdirektiv:

gcc -I/usr/java/jdk1.4/include -I/usr/java/jdk1.4/include/linux -fPIC Segment.cc -c

gcc -shared -Wl,-soname,libAioImage.so -o libAiboImage.so Segment.o

*/

  // the segmentation class
  Segment segm;

  // image data
  byte gBufY[6336];
  byte gBufU[6336];
  byte gBufV[6336];


JNIEXPORT jint JNICALL 
Java_AiboImage_setImage(JNIEnv *env, jobject obj, jintArray _y, jintArray _u, jintArray _v) 
{
  // find size of array
  jsize ylen = env->GetArrayLength(_y); // all y,u,v are same size, so no need for three sizes...

  // extract the arrays from their java wrappers...
  int *yData = (env->GetIntArrayElements(_y, 0));
  int *uData = (env->GetIntArrayElements(_u, 0)); 
  int *vData = (env->GetIntArrayElements(_v, 0));

  // make byte arrays from int arrays...
  for(int x = 0; x < ylen; x++) {
    gBufY[x] = (byte)(yData[x] & 0xff);
    gBufU[x] = (byte)(uData[x] & 0xff);
    gBufV[x] = (byte)(vData[x] & 0xff);
  }

  // and point the buffers to this data
  segm.bufY = (byte *)gBufY;
  segm.bufU = (byte *)gBufU;
  segm.bufV = (byte *)gBufV; 

  return 0;
}

JNIEXPORT jint JNICALL 
Java_AiboImage_setThreshold(JNIEnv *env, jobject obj, jintArray threshold) 
{  
  int *thres = (env->GetIntArrayElements(threshold, 0));

  // set the thresholds directly
  for(int x = 0; x < 8; x++)
    segm.threshold[x] = thres[x];
  
  return 0;
}

JNIEXPORT jintArray JNICALL 
Java_AiboImage_getSegmentation(JNIEnv *env, jobject obj, jintArray chImg) 
{
  jintArray segmentedData = env->NewIntArray(6336); // create return array

  // find size of array
  jsize chlen = env->GetArrayLength(chImg); // get size
  
  // two temporary buffers for byte/int int/byte conversion
  int tempBuff[6336];
  byte tmpBuff[chlen];


  // extract the array from its java wrapper...
  int *tempBuf = (env->GetIntArrayElements(chImg, 0));

  // make byte array from int array...
  for(int x = 0; x < chlen; x++) {

    tmpBuff[x] = 0x0;

    if (tempBuf[x] == 1)
      tmpBuff[x] = 0x1;
    if (tempBuf[x] == 2)
      tmpBuff[x] = 0x2;
    if (tempBuf[x] == 3)
      tmpBuff[x] = 0x4;
    if (tempBuf[x] == 4)
      tmpBuff[x] = 0x8;
    if (tempBuf[x] == 5)
      tmpBuff[x] = 0x10;
    if (tempBuf[x] == 6)
      tmpBuff[x] = 0x20;
    if (tempBuf[x] == 7)
      tmpBuff[x] = 0x40;
    if (tempBuf[x] == 8)
      tmpBuff[x] = 0x80;


  }

  // do the segmentation
  segm.CheckChannels(tmpBuff, gBufY, gBufU, gBufV, 1);

  // change it back to int
  for(int x = 0; x < 88; x++) {
    for(int y = 0; y < 72; y++) {
      tempBuff[y * 88 + x] = (int)segm.binImage[y * 88 + x];
    }
  }
  
  env->SetIntArrayRegion(segmentedData, 0, 6335, (jint *)tempBuff);
  
  return segmentedData;  
}
