import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

public class HelpWindowFrame extends JFrame {
    private JPanel contentPane;

    private TextArea helpText = new TextArea("The Toolbox\n\nHere you will find the most commonly used tools and operations. The toolbox resides in a window of its own, and can easily be placed anywhere on the screen you find convenient.\n\nLoad - Opens the load image dialog which lets you select and load one or more images from your hard drive or directly from your Memory Stick reader. Images loaded are displayed in the image index window on the bottom of your screen.\n\nSave CT - Opens the save color table dialog which lets you specify a filename and path to save your color table. You will be prompted for how to label the color table arrays, just press ok to save it as generic. For more information on the format color tables are saved in.\n\nUndo - Lets you erase you last selected pixel from your color table. Undo works with channels, so only pixels selected in the color currently active is erased. To undo a selection in another color, simply select that color in the toolbox and press undo the desired number of times. There are twenty steps of undo in each channel. Please note that the results from undo will not be visible until the image window is in focus. Undo effects all images, not just the one(s) currently displayed.\n\nShow Blobs - Checking this option displays all the images in the index window as segmented. This is a great way to get an overview of how good your color table is at the moment. Please note that keeping this option checked at all time might drastically reduce program performance.\n\nShow In CT - Checking this option lets you click a pixel in an image and see where in the color space it is located. This is a powerful tool to find stray pixels in you color table.\n\nColor palette - Clicking on these selects which color you are currently working with. The button is colored in the active color. This also affects which undo channel is used. \n\nThreshold - This slider show the threshold value for the current color. Just drag it to change the threshold. Thresholds affect how many pixels will be included when growing the segments. Thresholds are saved with the color table.\n*****************\n\nThe Image window\n\nThe image window is the main working window. In this window you click to select pixels (color values) to be included in your color table. The small display to the right shows the segmented version of the image. Clicking on the buttons with arrows displays the next/previous image.\n*****************\n\nMenus\n\nThe menus contain many options not available anywhere else in ColorCalibration. There are five main categories: File, Color Table, Options, Window and Help.\n\nFile - Image - Load - Opens the load image dialog which lets you select and load one or more images from your hard drive or directly from your Memory Stick reader. Images loaded are displayed as thumbnails in the image index window on the bottom of your screen. Single click on a thumbnail to open it up in a new window.\n\nFile - Color Table - Load - Opens the load color table dialog which lets you load a previously saved color table.\n\nFile - Color Table - Save - Opens the save color table dialog which lets you specify a filename and path to save your color table. You will be prompted for how to label the color table arrays, just press ok to save it as generic.\n\nFile - Preferences - Opens the preferences dialog.\n\nFile - Exit - Exits the program.\n\nColor Table - Modify - Opens the color table window. This lets you see how the color values in your color table are distributed in the color space.\n\nColor Table - Reset - Resets the color table. Please be careful with this function. You must undo for all the color channels to get your color table back.\n\nOptions - Automatization - Opens the automatization window.\n\nWindow - Show Toolbox - Opens the toolbox window if it is not already on open.\n\nWindow - Show Image Index - Opens a new image index.\n\nHelp - About - Shows info and help about ColorCalibration.\n*****************\n\nImage index\n\nThe image index window shows small, thumbnail size, versions of all images loaded. Images can be displayed as regular images or segmented ones, based on the current color table. Just check Show Blobs in the toolbox to show segmented images. Please note that updating many segmented images might drastically reduce program performance.\n*****************\n\nColor tables\n\nThe color table window shows how the color values in the images loaded and in the current color table are distributed in the YUV color space. There are 32 levels of Y (light), which one is displayed is indicated by the light slider. For each Y value, the U and V values in the current color table are shown by a box in the color space area. If the show pixels option is checked, all color values that are actually in the images are also plotted in each box, with darker colors representing a higher density. Boxes showing the color value boundaries may overlap each other, and in that case the color with the highest priority takes precedence. The colors are ordered in the same order they are displayed in the toolbox palette, with the default order being orange, yellow, sky blue, pink, green, red, dark blue, white, carpet, black and silver where orange has the highest priority. Priority rules are the same for the show pixels option, in case of overlapping.\n\nWhen the mouse is moved around in the color space area the color directly under the mouse pointer is displayed to the right. Also displayed to the right are the actual U- and V values for the color currently selected. The button Clear color clears the color boundaries for the color currently selected and only for the current Y (light) value. At all times you need to press the Update CT buttons for the changes to take effect.\n\nClicking the button Close closes the color table window.\n*****************\n\nAutomatization\n\nThe automatization window lets you specify whether to run the automatization process with the current color table or one loaded from disk. Clicking Configure lets you specify how many color values to select during the automatization and if you want to use local histograms (e.g. one histogram for each image and color values are selected from each) or global histograms (e.g. one histogram for the entire set of images to select color values from) or both. It also lets you specify how many color values of each color you want to add to you color table.\n\nNote that a low figure is recommended if using local histograms. With the global histograms a larger figure is probably better.\n\nExperiment to find values that work for your type of playing field and light conditions.\n\nThe automatization process needs a large set of images to be effective.\n*****************\n\nPreferences\n\nThe preferences window lets you set your colors and their names. It also lets you check/uncheck colors. Unchecking a color makes ColorCalibration ignore the color for all processes, e.g. segmentation and automatization. However, you are still able to select pixels from an image with an unchecked color; it just wont show in the segmentation.\n\nPreferences also let you specify a system library that contains segmentation routines, if you want to use your own segmentation routines. Just specify the name, and make sure the systems library path environment variable is set to where the segmentation library is located and check the option Use native to use your own segmentation routines.\n*****************\n\nPGM file format\n\nThe PGM file format is the format the tutorial Open-R program Image Capture. It is divided in three files; the Y-, U- and V component. No compression\n*****************\n\nKJI file format\n\nThe KJI file format is, unlike PGM, a one file per image format. It is a YUV format and bears a close resemblance with the PGM format. No compression.\n*****************\n\nColor Table file format\n\nThe color tables are saved as c++ source code, giving you the ability to just include it when you compile your Open-R source code\n*****************\n\nCompiling the code to work with ColorCalibration\n\nIn order to use your own segmentation routines in ColorCalibration, you first need to compile your segmentation code into a system library with a small ColorCalibration wrapper included. A wrapper working on Team Swedens segmentation routines is included in the ColorCalibration distribution. Modify it to work with your segmentation code and compile it.\n\nExample for linux:\n\ng++ -I/usr/java/jdk1.4/include -I/usr/java/jdk1.4/include/linux -fPIC Segm.cc -c\n\ng++ -shared -Wl,-soname,libMySegm.so -o libMySegm.so Segm.o\n\nWhere Segm.cc is your segmentation routine with the ColorCalibration wrapper included:\n\n#include 'AiboImage.cc'\n\nThe wrapper contains three methods. You may not change the input or return parameters of these methods. But you may freely alter the methods to fit your segmentation code.\n\n/* set the image from Y-, U- and V-arrays */\njint Java_AiboImage_setImage(JNIEnv*, jobject, jintArray, jintArray, jintArray);\n\n/* set the threshold from an array of thresholds */\njint Java_AiboImage_setThreshold(JNIEnv*, jobject, jintArray);\n\n/* get the segmentation from array with colors indexes matching color table */\njintArray Java_AiboImage_getSegmentation(JNIEnv*, jobject, jintArray);", 40, 500, TextArea.SCROLLBARS_VERTICAL_ONLY);

    public HelpWindowFrame() {
	enableEvents(AWTEvent.WINDOW_EVENT_MASK);
	try {
	    helpInit();
	}
	catch(Exception e) {
	    JOptionPane.showMessageDialog(null, "An error occured and ColorCalibration will now close\nSorry");
	    System.exit(0);
	}
    }
    
    private void helpInit() {
	contentPane = (JPanel)this.getContentPane();
	this.setSize(440, 345);
	this.setTitle("Help");
	
	helpText.setBounds(new Rectangle(0, 0, 430, 335));

	contentPane.add(helpText,null);
    }
}

